<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Monolog\Handler\StreamHandler;

class LocationCloneCommand extends Command
{
    protected $signature = 'sync:locations';
    protected $description = 'Sync tables from server to client';

    public function handle()
    {

        try {
        // Log success message
            Log::channel('custom-command-log')->info('Locations Command Initiated successfully.');
        $allowed_tables = ["countries","counties","regions","cities","areas"];

        foreach ($allowed_tables as $tableName) {
            // Access the API endpoint
            $response = Http::get(env('CONSOLE_API') . 'get-location-clone/' . $tableName);


            if ($response->successful()) {
                $data = $response->json();

                // Drop table if exist statement
                DB::statement("DROP TABLE IF EXISTS $tableName");
                $this->info("Table Dropped If Exist\n");

                // Run the create table statement
                DB::statement($data['create_table_statement']);
                $this->info("Create Statements executed successfully\n");

                // Run the insert statement
//                DB::statement($data['insert_statement']);
                $this->insertOrUpdate($tableName,$data['table_data']);
                $this->info("Insert Statements executed successfully\n");

            } else {
                $this->error("Locations Error accessing API endpoint: {$response->status()}");
            }
        }

        } catch (\Exception $e) {
            // Log any exception or error that occurred
            Log::channel('custom-command-log')->info('Locations Error executing command: ' . $e->getMessage() );

        }finally {
            // Create a new log file for the next command execution
            $this->rotateLogFile();
        }
    }

    private function insertOrUpdate($table,array $rows){

        if(count($rows) <= 0){
            return true;
        }
        $insert_data = DB::table($table)->insert($rows);

        return $insert_data;
    }

    private function rotateLogFile()
    {
        Log::channel('custom-command-log')->info('Locations Command completed successfully on - '. now()->format('Y-m-d_H-i-s') );
    }
}
