<?php
namespace App\Http\Helpers;

use App\Models\Field;
use App\Models\Form;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Artisan;
use \Illuminate\Support\Facades\Schema;
use Illuminate\Support\Str;


class FormGenerator
{
    protected $rules = array();

    public function generate_form($form_id,$parent_table=null)
    {
        $form = Form::find($form_id);
        $form_fields = $form->fields;
        $table = Str::plural($form->table_name);
        if ($parent_table == null){
            $table_name = Str::plural($form->table_name);
        }
        else{
            $table_name = Str::plural($parent_table).'_'.Str::plural($form->table_name);
        }

        if (!Schema::hasTable($table_name)) {
            Schema::create($table_name, function (Blueprint $table) use ($form_fields,$parent_table) {
                $table->increments('id');
                if ($parent_table != null){
                    $table->integer($parent_table.'_id');
                }
                foreach ($form_fields as $form_field) {
                    $this->setFormFields($form_field, $table);
                }
                $table->timestamps();
            });
            $command = 'make:model';
            $params = [
                'name' => Str::studly(Str::singular($table_name)),
            ];

            //CREATES A MODEL FOR THE PARENT FORM TABLE IN THE DEFAULT MODELS DIRECTORY
            Artisan::call($command, $params);
        }
        else {
            $columns = Schema::getColumnListing($table);

            Schema::table($table_name, function (Blueprint $table) use ($form_fields,$columns) {
                foreach ($form_fields as $form_field) {

                    if (in_array($form_field->name,$columns)) {
                        $this->updateFormFields($form_field, $table);
                    }
                    else {
                        $this->setFormFields($form_field, $table);
                    }

//                    if (($key = array_search("green", $colors)) !== false) {
//                        unset($colors[$key]);
//                    }
                }
            });
        }

        if ($form->sub_forms->isNotEmpty()) {
            $subforms = $form->sub_forms;
            foreach($subforms as $form){
                $this->generate_form($form->form_id,$table);
            }
        }
    }

    /**
     * @param Field $form_field
     * @param Blueprint $table
     */
    public function setFormFields(Field $form_field,Blueprint $table)
    {
        if (in_array($form_field->input_type, ['checkbox', 'file', 'select'])) {

//            if ($form_field->is_multiple) {
                $table->text($form_field->name)->nullable();
//            }
//            else {
//                $table->string($form_field->name)->nullable();
//            }

        } elseif (in_array($form_field->input_type, ['color', 'radio', 'email', 'password', 'text', 'numeric_text', 'alpha_numeric'])) {

            $table->string($form_field->name)->nullable();

        } elseif (in_array($form_field->input_type, ['number'])) {

            $table->float($form_field->name)->nullable();

        } elseif (in_array($form_field->input_type, ['tel'])) {

            $table->string($form_field->name, 20)->nullable();

        } elseif (in_array($form_field->input_type, ['textarea'])) {

            $table->text($form_field->name)->nullable();

        } elseif (in_array($form_field->input_type, ['dateTime'])) {

            $table->dateTime($form_field->name)->nullable();

        } elseif (in_array($form_field->input_type, ['date','dob'])) {

            $table->date($form_field->name)->nullable();

        } elseif (in_array($form_field->input_type, ['time'])) {

            $table->time($form_field->name)->nullable();

        }
    }

    /**
     * @param Field $form_field
     * @param Blueprint $table
     */
    public function updateFormFields(Field $form_field,Blueprint $table)
    {
        if (in_array($form_field->input_type, ['checkbox', 'file', 'select'])) {

//            if ($form_field->is_multiple) {
                $table->text($form_field->name)->nullable()->change();
//            }
//            else {
//                $table->string($form_field->name)->nullable()->change();
//            }

        } elseif (in_array($form_field->input_type, ['color', 'radio', 'email', 'password', 'text', 'numeric_text', 'alpha_numeric'])) {

            $table->string($form_field->name)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['number'])) {

            $table->float($form_field->name)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['tel'])) {

            $table->string($form_field->name, 20)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['textarea'])) {

            $table->text($form_field->name)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['dateTime'])) {

            $table->dateTime($form_field->name)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['date','dob'])) {

            $table->date($form_field->name)->nullable()->change();

        } elseif (in_array($form_field->input_type, ['time'])) {

            $table->time($form_field->name)->nullable()->change();

        }
    }
}
