<?php

namespace App\Http\Controllers;

use App\Http\Traits\MembershipTrait;
use App\Http\Traits\HttpResponses;
use App\Http\Traits\ModelTracker;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\RecordTracker;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class AdminApiController extends Controller
{
    use HttpResponses,MembershipTrait,ModelTracker;

    public $successStatus = 200;
    public $noContentStatus = 204;
    public $badRequestStatus = 400;
    public $invalidAuthStatus = 401;
    public $hiddenFieldColumns = ['created_at','updated_at','workflow_actors'];
    public $hiddenUserColumns = ['email_verified_at','deleted_at','password','remember_token','created_at','updated_at'];

    public function dashboardStatistics()
    {
        if (!auth()->user()->is_admin) {
            return $this->errorResponse(null,'Access Denied',401);
        }

        $applicants = Applicant::count();

        $active_claimers = DB::table('applications')
            ->distinct()
            ->where('status','=','Approved')
            ->where('is_beneficiary','=',1)
            ->pluck('applicant_number')
            ->toArray();

        $beneficiaries_by_region = DB::table('applicants')
            ->where('applicants.status','=',1)
            ->whereIn('regID',$active_claimers)
            ->select(DB::raw('count(id) as beneficiaries, place_of_births as region'))
            ->groupBy('place_of_births')
            ->pluck('beneficiaries','region')->toArray();


        $new_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('benefits.archived', '=', 0)
            ->whereIn('applications.status', ['Submitted','Resubmitted'])
            ->count();
        $active_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('applications.archived','=',0)
            ->whereIn('applications.status', ['Approved'])
            ->count();


        /*
        **  Get Year at a Glance Values
        */
        $active_benefits = DB::table('benefits')
            ->leftJoin('applications','benefit_id','=','benefits.id')
            ->select(DB::raw('count(*) as total_applications, benefits.titles as benefit_title'))
            ->where('applications.archived','=',0)
            ->groupBy('benefits.titles')
            ->orderBy('benefits.titles','ASC')
            ->pluck('total_applications','benefit_title')->toArray();
        $total_count_apps_benefit = null;
//        $apps_benefit_titles = null;

        $count = 1;
        foreach($active_benefits as $key => $value) {
//            $total_count_apps_benefit[] = (int)$value;
//            $apps_benefit_titles[] = (string)$key;
            $total_count_apps_benefit[(string)$key] = (int)$value;
        }

        /*
        **  Get Benefit by Gender distribution
        */
        $applications = Application::active()->with(['applicant'])->get();
        $children = config('settings.age_groups.children');
        $youth = config('settings.age_groups.youth');
        $aged = config('settings.age_groups.aged');
        $childrenTotal = $youthTotal = $agedTotal = $male = $female = 0;
        $age_groups = $age_groups_values = $gender_values = $gender_groups = array();
        $total_applications = $applications->count();

        //Looping through the applications to get applicants information
        foreach ($applications as $application) {
            $applicant = $application->applicant;

            //Checking and collecting age group data of applicants
            $applicant_age = getRawAgeByDate($applicant->dobs);

            if ($applicant_age <= $children[1]) {
                $childrenTotal += 1;
            }
            elseif ($applicant_age <= $youth[1]){
                $youthTotal += 1;
            }
            else {
                $agedTotal += 1;
            }

            //Checking and collecting gender data of applicants
            if ($applicant->genders == 'Male'){
                $male += 1;
            }
            else {
                $female += 1;
            }
        }

        // Collate gender data into a string
        if ($male  > 0){
//            $gender_groups[] = 'Male ('.round($male*100/$total_applications).'%)';
//            $gender_values[] = $male;
            $gender_groups['Male ('.round($male*100/$total_applications).'%)'] = $male;
        }
        if ($female > 0){
//            $gender_groups[] = 'Female ('.round($female*100/$total_applications).'%)';
//            $gender_values[] = $female;
            $gender_groups['Female ('.round($female*100/$total_applications).'%)'] = $female;
        }

        //Collate the age groups data into a string
        if ($childrenTotal > 0) {
//            $age_groups[] = 'Children ('.round($childrenTotal*100/$total_applications).'%)';
//            $age_groups_values[] = $childrenTotal;
            $age_groups['Children ('.round($childrenTotal*100/$total_applications).'%)'] = $childrenTotal;
        }
        if ($youthTotal  > 0){
//            $age_groups[] = 'Youth ('.round($youthTotal*100/$total_applications).'%)';
//            $age_groups_values[] = $youthTotal;
            $age_groups['Youth ('.round($youthTotal*100/$total_applications).'%)'] = $youthTotal;
        }
        if ($agedTotal  > 0){
//            $age_groups[] = 'Aged ('.round($agedTotal*100/$total_applications).'%)';
//            $age_groups_values[] = $agedTotal;
            $age_groups['Aged ('.round($agedTotal*100/$total_applications).'%)'] = $agedTotal;
        }

        return $this->successResponse([
            'applicants'=>$applicants,
            'new_applications'=>$new_applications,
            'active_applications'=>$active_applications,
            'beneficiaries_by_region'=>$beneficiaries_by_region,
            'total_count_applications_benefit'=>$total_count_apps_benefit,
            'gender_groups'=>$gender_groups,
            'age_groups'=>$age_groups,
        ],'Dashboard Statistics');
    }
}
