<?php

namespace App\Http\Controllers;

use App\Models\Audit;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\Office;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;

use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

class AjaxController extends Controller
{
    protected function okclicked(){
        $email = session('email');
        $password = session('pwd');

        $login_data =[
            'email' => $email,
            'password' => $password
        ];

        if (Auth::attempt($login_data)){
            session()->regenerate();
            if(env('ALLOW_MULTI_LOGIN') == 'off')
            {
                Auth::logoutOtherDevices($password);
            }
            //record login in audits table
            $data = [
                'auditable_id' => auth()->user()->id,
                'auditable_type' => "Logged In",
                'event'      => "Logged In",
                'url'        => request()->fullUrl(),
                'ip_address' => request()->getClientIp(),
                'user_agent' => request()->userAgent(),
                'created_at' => Carbon::now(),
                'updated_at' => Carbon::now(),
                'user_id'    => auth()->user()->id,
            ];
            Audit::create($data);




            return redirect()->route('dashboard');
        }
    }
    protected function deactivateUser(Request $request){

        $user = User::where('email', $request->the_email)->first();

        $user->deactivated_at = date('Y-m-d H:i:s');
        $user->save();
        return back();

    }
    protected function activateUser(Request $request){

        $user = User::where('email', $request->user_email)->first();
        $user->deactivated_at = null;
        $user->save();
        return back();

    }
    public function deleteUser(Request $request)
    {
        $user = User::where('email', $request->user_email_del)->first();
        if($user->deactivated_at){
            $user->deleted_at = date('Y-m-d H:i:s');
            $user->deleted_by = Auth::id();

        }else{
            $user->deactivated_at = date('Y-m-d H:i:s');
            $user->deleted_at = date('Y-m-d H:i:s');
            $user->deleted_by = Auth::id();

        }
        $user->save();
        return back();


    }
    public function recoverUser(Request $request)
    {
        $user = User::where('email', $request->user_email_recover)->first();

        $user->deleted_at = null;
        $user->deleted_by = null;
        $user->deactivated_at = null;
        $user->save();
        return back();

    }
    public function fetchProfile($id){
        $profile = User::find($id);
        return response()->json($profile);

    }

    public function fetchOffice($id)
    {
        $office = Office::find($id);
        return response()->json($office);

    }


    public function getOffice($id)
    {
        $office = Office::find($id);
        return response()->json(['name'=>$office->name,'code'=>$office->code,'coverable_date'=>$office->coverable_date,'parent'=>$office->parent?->name]);
    }


    public function deactivateLookup(Request $request)
    {
        $lookup = LookupCode::find($request->lookup_id);

        $lookup->activeFlag = 0;
        $lookup->save();
        return response()->json(['success'=>'done']);
    }

    public function activateLookup(Request $request)
    {
        $lookup = LookupCode::find($request->lookup_id);

        $lookup->activeFlag = 1;
        $lookup->save();
        return response()->json(['success'=>'done']);
    }

    public function getOffices($countryId)
    {
        $offices = Office::where('location_id', $countryId)->get();

        return response()->json($offices);
    }

    public function getOptions($id)
    {
        $options = Lookup::where('lookup_code_id', $id)->get();

        return response()->json(['options' => $options]);
    }

    public function getOptionDetails($id){
        $option = Lookup::find($id);
        return response()->json($option);

    }

    public function getHigherOffices($id)
    {
        $selected_office_type = Lookup::find($id);

        //get office types
        $office_type_look_up_id = LookupCode::where('lookupName', 'Office Type')->first()?->id;

        //get higher office types
        $higher_office_types = Lookup::where([['lookup_code_id', $office_type_look_up_id], ['activeFlag', 1], ['order', '<',  $selected_office_type->order]])->get();

        $ids = []; // Initialize an empty array to store ids

        foreach ($higher_office_types as $higher_office_type) {
            $ids[] = $higher_office_type->id; //
        }

        $higher_offices = Office::whereIn('type', $ids)->get();

        return response()->json(['offices' => $higher_offices]);


    }

    public function getHigherRoles($rolesId)
    {

        $roles = Role::findOrFail($rolesId)->getHigherRoles();

        return response()->json($roles);
    }

    public function getRoleSupervisors($roleId)
    {

        $selected_role = Role::find($roleId);

        if($selected_role->lead == null){
            $supervisors = [];
        }else {
            $supervisors = User::query()
                ->whereHas('roles',function ($query) use($selected_role){
                    $query->where('id',$selected_role->lead);
                })
                ->where('deactivated_at', null)
                ->get();
        }

        return response()->json(['supervisors'=>$supervisors]);

    }

}
