<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Mail\NewUserAccount;
use App\Models\Customer;
use App\Models\LookupCode;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;

class CustomerRegisterController extends Controller
{

    public function customerSignupForm()
    {
        $gender_list = LookupCode::activeOptions(config('settings.gender_code'));
        return view('auth.customer-signup', compact('gender_list'));
    }

    public function create(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|string',
            'firstname' => 'required|string',
            'phone' => 'required|string|max:15',
            'call_code' => 'required|string',
            'country' => 'required|numeric',
        ]);

        $existing_phone = User::where('phone',cleanString($request->phone))->where('call_code',$request->call_code)->count();
        if($existing_phone > 0) {
            session()->flash('status', 'error');
            session()->flash('message', 'This phone number address already exists');
            return back()->withInput();
        }
        if(isset($request->email)) {
            $existing_email = User::where('email',$request->email)->count();
            if($existing_email > 0) {
                session()->flash('status', 'error');
                session()->flash('message', 'This email address already exists');
                return back();
            }
        }

        $code = strtoupper(Str::random(4));

        $user = new User();
        $user->name =  $request->name;
        $user->firstname =  $request->firstname;
        $user->phone =  cleanString($request->phone);
        $user->api_token =  Str::random(60);
        $user->is_admin =  false;
        $user->country_id =  $request->country;
        $user->call_code =  $request->call_code;
        $user->password =  Hash::make($code);
        $user->verification_code =  $code;
        $user->email_verified_at = now();
        $user->last_instance_opened = 0;

        if ($user->save()) {
//            $user->roles()->sync(env('DEFAULT_ROLE'));
            $user->roles()->sync([env('DEFAULT_ROLE',7) => ['instance_id' => 0]]);
            $message = 'Thank you! You have received a 4-digits code via sms for verification.';
            $sms = env('APP_NAME').": Your account has been created. Use this verification code - $code as your password to activate your account. Thank you.";
            sendSMS(cleanString($request->phone),$sms);

            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('login');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry please try again');
            return back();
        }
    }

    public function verify()
    {
        return view('auth.customer-verify');
    }

    public function verifyCode(Request $request)
    {
        $this->validate($request, [
            'mobile' => 'required|string',
            'verification_code' => 'required|string|min:4|max:4',
        ]);

        $customer = User::where('phone',cleanString($request->mobile))->where('verification_code',$request->verification_code)->first();

        if ($customer) {
            $customer->verification_code = null;
            $customer->account_verified_at = now();
            $customer->email_verified_at = now();
            $customer->status = 1;
            $customer->save();
            $message = 'Your account has been verified.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('login');
        }
        else {
            $message = "Sorry! Invalid mobile number or code.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

    }

}
