<?php


namespace App\Http\Controllers\Auth;


use App\Http\Controllers\Controller;
use App\Http\Middleware\LoadUserPermissions;
use App\Models\ToastNotification;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;

class ExpiredPasswordController extends Controller
{

    public function __construct() {

        $this->middleware( LoadUserPermissions::class );
    }

    public function expired()
    {
        $user = Auth::user();
        return view('auth.passwords.expired',compact('user'));
    }

    public function postExpired(Request $request)
    {
        //dd(bcrypt($request->old_password));

        $this->validate($request,[
            'old_password' => 'required|string|min:6',
            'new_password' => 'min:6|required_with:password_confirmation|same:password_confirmation',
            'password_confirmation' => 'required|string|min:6'
        ]);


        // Checking current password
        if (!Hash::check($request->old_password, $request->user()->password)) {
            return redirect()->back()->withErrors(['old_password' => 'Current password is not correct']);
        }

        $request->user()->update([
            'password' => bcrypt($request->new_password),
            'password_changed_at' => Carbon::now()->toDateTimeString()
        ]);



        notify( new ToastNotification( 'Password Changed', 'Your expired password has been changed to the new password successfully', 'success', 'right' ) );

        return redirect()->back();

    }
}
