<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Auth\Traits\AuthenticatesUsers;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/dashboard';
    protected $maxAttempts = 2; // Default is 5
    protected $decayMinutes = 1; // Default is 1


    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware(['guest:web'])->except('logout');
    }

    protected function authenticated(Request $request, $user)
    {
        Auth::logoutOtherDevices($request->password);
        Auth::user()->last_login = date('Y-m-d H:i:s');


        if (auth()->user()->hasRole('beneficiary')) {
            return  redirect()->route('customer.profile');
        }

        if (is_null(getActiveInstance())) {
            $deployment = auth()->user()->deployments()->first();
            if ($deployment) {
                auth()->user()->last_instance_opened = $deployment->id;
            }

        }
        Auth::user()->save();

        if (auth()->user()->hasRole('owner') && !auth()->user()->hasOrganisation()) {
            return  redirect()->route('on_boarding.index');
        }

        if (auth()->user()->onboardingStage && auth()->user()->hasOrganisation()) {
            return  redirect()->route('on_boarding.index');
        }
    }

    /**
     * Get the login username to be used by the controller.
     *
     * @return string
     */
    public function username()
    {
        if(is_numeric(cleanString(request()->phone))) {
            return 'phone';
        }
        else {
            return 'email';
        }
    }
    public function credentials(Request $request)
    {
        if ($this->username() == 'phone') {
            $credentials = $request->only($this->username(), 'password','call_code');
        }
        else {
            $credentials = $request->only($this->username(), 'password');
        }

        return $credentials;
    }
}
