<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Auth\Traits\RegistersUsers;
use App\Models\OnboardingStage;
use App\Models\User;
use App\Http\Controllers\Controller;
use \Illuminate\Support\Facades\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = '/';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'firstname' => 'required|string|max:255',
            'surname' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6|confirmed',
            'terms_acceptance' => 'required',
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {
        if ($data['user_type'] == 'organization') {
            $user = User::create([
                'firstname' => $data['firstname'],
                'name' => $data['surname'],
                'email' => $data['email'],
                'password' => Hash::make($data['password']),
                'is_admin' =>  1,
                'user_type' =>  'owner',
            ]);
//            $user->roles()->sync(env('OWNER_ROLE',3));
            $user->roles()->sync([env('OWNER_ROLE',3) => ['instance_id' => 0]]);

            OnboardingStage::UpdateOrCreate([
                'user_id'   =>  $user->id
            ],[
                'stage' =>  'organization_setup'
            ]);
        }
//        else if($data['user_type'] == 'employee'){
//            $user = User::create([
//                'firstname' => $data['firstname'],
//                'name' => $data['surname'],
//                'email' => $data['email'],
//                'password' => Hash::make($data['password']),
//                'is_admin' =>  0,
//                'user_type' =>  'user',
//            ]);
//            $user->roles()->sync(env('DEFAULT_ROLE'));
//        }
        else{
            $user = User::create([
                'firstname' => $data['firstname'],
                'name' => $data['surname'],
                'email' => $data['email'],
                'password' => Hash::make($data['password']),
                'is_admin' =>  0,
                'user_type' =>  'user',
                'last_instance_opened' =>  0,
            ]);
//            $user->roles()->sync(env('DEFAULT_ROLE'));
            $user->roles()->sync([env('DEFAULT_ROLE',7) => ['instance_id' => 0]]);
        }

        return $user;
    }

	protected function guard() {
		return Auth::guard('web');
	}

    public function signUp(Request $request)
    {
        return view('auth.signup');
    }
}
