<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Mail\NewUserAccount;
use App\Models\Audit;
use App\Models\OnboardingStage;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use Laravel\Socialite\Facades\Socialite;

class SocialMediaConnections extends Controller
{
    public function handleProviderCallback()
    {
        $state = request()->state;
        parse_str($state, $result);
        return $result['sign_type'];
    }

    public function redirect($sign_type=null)
    {
        if (is_null($sign_type)) return Socialite::driver('google')->redirect();
            return Socialite::driver('google')->with(['state'=>'sign_type='.$sign_type])->redirect();
    }

    public function callback()
    {
        $signed_user = Socialite::driver('google')->stateless()->user();

        $user = User::where('email','=',$signed_user->email)->first();

        $route = 'dashboard';

        if (!$user) {

            $sign_type = $this->handleProviderCallback();

            if ($signed_user->user['verified_email']) {
                $verified = now();
            }

            if ($sign_type == 'beneficiary') {
                $code = strtoupper(Str::random(4));

                $user = User::updateOrCreate([
                    'email' => $signed_user->user['email'],
                ], [
                    'name' => $signed_user->user['family_name'],
                    'firstname' => $signed_user->user['given_name'],
                    'profile_photo_path' => $signed_user->user['picture'],
                    'user_type' => 'user',
                    'is_admin' => false,
                    'password' => Hash::make($code),
                    'verification_code' => $code,
                    'email_verified_at' => $verified,
                ]);
//                $user->roles()->sync(env('DEFAULT_ROLE'));
                $user->roles()->sync([env('DEFAULT_ROLE',7) => ['instance_id' => 0]]);

                if($user->email != null){
                    Mail::to([$user->email])->send(new NewUserAccount($code));
                }

                session()->flash('status', 'success');
                session()->flash('message', 'Welcome to Alleviate! Please check your inbox for your verification code.');
                $route = 'customer.profile';

            }
            elseif($sign_type == 'organization') {
                $user = User::updateOrCreate([
                    'email' => $signed_user->user['email'],
                ], [
                    'name' => $signed_user->user['family_name'],
                    'firstname' => $signed_user->user['given_name'],
                    'profile_photo_path' => $signed_user->user['picture'],
                    'user_type' => 'owner',
                    'is_admin' => 1,
                    'password' => Hash::make('Pa$$w0rd1'),
                    'email_verified_at' => $verified,
                ]);
//                $user->roles()->sync(env('OWNER_ROLE',3));
                $user->roles()->sync([env('OWNER_ROLE',3) => ['instance_id' => 0]]);

                OnboardingStage::UpdateOrCreate([
                    'user_id'   =>  $user->id
                ],[
                    'stage' =>  'organization_setup'
                ]);
                session()->flash('status', 'success');
                session()->flash('message', 'Welcome to Alleviate! You are a few steps away from touching lives.');
                $route = 'on_boarding.index';
            }
            else{
                session()->flash('status', 'error');
                session()->flash('message', 'No records found. Sign up to enjoy Alleviate');
                return redirect()->route('login');
            }

        }

        Auth::login($user);

        if (Auth::check()) {
            session([
                'my_permissions'    =>  Auth::user()->allPermissions()->pluck('name')->toArray(),
//                'my_roles'    =>  Auth::user()->roles()->pluck('name')->toArray(),
                'role'    =>  auth()->user()->roles()->pluck('name')->first(),
                'role_name'    =>  auth()->user()->roles()->pluck('display_name')->first(),
            ]);

            $data = [
                'auditable_id' => auth()->id(),
                'auditable_type' => "Logged In",
                'event'      => "Logged In",
                'url'        => request()->fullUrl(),
                'ip_address' => request()->getClientIp(),
                'user_agent' => request()->userAgent(),
                'created_at' => Carbon::now(),
                'updated_at' => Carbon::now(),
                'user_id'          => auth()->id(),
            ];

            //create audit trail data
            Audit::create($data);
        }

        return redirect()->route($route);
    }
}
