<?php
/**
 * Created by PhpStorm.
 * User: stephen
 * Date: 15/03/18
 * Time: 12:10
 */

namespace App\Http\Controllers\Auth\Traits;


use App\Models\Audit;
use Carbon\Carbon;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

trait RegistersUsers
{
	// use RedirectsUsers;

	/**
	 * Show the application registration form.
	 *
	 * @return \Illuminate\Http\Response
	 */
	public function showRegistrationForm()
	{
		return view('auth.register');
	}

	/**
	 * Handle a registration request for the application.
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @return \Illuminate\Http\Response
	 */
	public function register(Request $request)
	{
		$this->validator($request->all())->validate();

		event(new Registered($user = $this->create($request->all())));

        $this->guard()->login($user);

		if (Auth::check()) {
            session([
                'my_permissions'    =>  Auth::user()->allPermissions()->pluck('name')->toArray(),
//                'my_roles'    =>  Auth::user()->roles()->pluck('name')->toArray(),
                'role'    =>  auth()->user()->roles()->pluck('name')->first(),
                'role_name'    =>  auth()->user()->roles()->pluck('display_name')->first(),
            ]);

            $data = [
                'auditable_id' => auth()->id(),
                'auditable_type' => "Logged In",
                'event'      => "Logged In",
                'url'        => request()->fullUrl(),
                'ip_address' => request()->getClientIp(),
                'user_agent' => request()->userAgent(),
                'created_at' => Carbon::now(),
                'updated_at' => Carbon::now(),
                'user_id'          => auth()->id(),
            ];

            //create audit trail data
            Audit::create($data);
        }

		return $this->registered($request, $user)
			?: redirect('/dashboard');
	}

	/**
	 * Get the guard to be used during registration.
	 *
	 * @return \Illuminate\Contracts\Auth\StatefulGuard
	 */
	protected function guard()
	{
		return Auth::guard('admin');
	}

	/**
	 * The user has been registered.
	 *
	 * @param  \Illuminate\Http\Request  $request
	 * @param  mixed  $user
	 * @return mixed
	 */
	protected function registered(Request $request, $user)
	{
		//
	}
}
