<?php

namespace App\Http\Controllers;

use App\Models\Client;
use App\Models\Deployment;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

class ClientController extends Controller
{
    public function index(Request $request)
    {
        /*
        $clients = Client::with(['owner','type'])->orderBy('name');

        if (isset($request->name)){
            $clients = $clients->where('name','like','%'.$request->name.'%');
        }

        if (isset($request->country) && $request->country != 'Select'){
            $clients = $clients->where('country_id','=', $request->country);
        }

        if (isset($request->business_type) && $request->business_type != 'Select'){
            $clients = $clients->where('field','=', $request->business_type);
        }

        $clients = $clients->paginate(50);
        */
        

        $deployments = Deployment::orderBy('name', 'asc')->get();

        return view('clients.index', compact('deployments'));
    }

    public function getClient($client_id)
    {
        $client = Client::find($client_id);

        if ($client) {
            $view = view('clients.client-form', compact('client'))->render();
        }
        else{
            $view = __('No record found');
        }
        return response()->json(['output'=>$view]);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveInfo(Request $request)
    {
        $client = Client::find($request->client_id);

        if ($client) {

            if(Client::where('name','=',$request->business_name)->where('id','!=',$request->client_id)->count() > 0) {
                session()->flash('status', 'error');
                session()->flash('message', 'Sorry! Organisation name already exists.');
                return back();
            }
            $client->name = $request->business_name;
            $client->field = $request->business_type;
            $client->country_id = $request->country;
            $client->status = 0;
            $client->save();

            session()->flash('status', 'success');
            session()->flash('message', 'Client information has been submitted for review.');
            return back();
        }
        else{
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid Client selected.');
            return back();
        }
    }

    public function newClient(Request $request)
    {
        if (!auth()->user()->isChannelPartner() && !auth()->user()->isNew()) {
            session()->flash('status', 'info');
            session()->flash('message', 'Subscription Required! Subscribe as a Channel Partner to allow you add multiple organisations.');
            return back();
        }
        if (!auth()->user()->hasActiveSubscription()) {
            session()->flash('status', 'error');
            session()->flash('message', 'Subscription Required! You have no active subscription as a Channel Partner. Subscribe Now');
            return back();
        }

        $request->validate([
            'business_name' => 'required|string|max:255|unique:clients,name',
            'business_type' => 'required|string|max:255',
            'country' => 'required|numeric|max:255',
        ],[
            'country.numeric'   =>  'Invalid country selected'
        ]);

        $client = new Client();
        $client->name = $request->business_name;
        $client->field = $request->business_type;
        $client->country_id = $request->country;
        $client->created_by = auth()->id();
        $client->status = 0;
        $client->admin = auth()->id();

        if ($client->save()) {
            auth()->user()->clients()->attach([$client->id]);

            if (is_null(auth()->user()->client_id)) {
                auth()->user()->client_id = $client->id;
                auth()->user()->save();
            }

            session()->flash('status', 'success');
            session()->flash('message', 'Your organisation has been submitted and pending verification');
            return redirect()->route('client.home');
        }
        else{
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Something went wrong.');
            return back();
        }
    }

    public function getOwner($client_id)
    {
        $client = Client::find($client_id);

        if ($client) {
            $view = view('clients.owner-form', compact('client'))->render();
        }
        else{
            $view = __('No record found');
        }

        return response()->json(['output'=>$view]);
    }

    public function changeOwner(Request $request)
    {
        $client = Client::find($request->client_id);

        if ($client && !is_null($client->owner)) {
            $client->admin = $request->user;

            if (auth()->user()->is_owner()) {
                if (isset($request->password)) {
                    if ($request->password != $request->password_confirmation) {
                        session()->flash('status', 'error');
                        session()->flash('message', 'The passwords do not match.');
                        return back();
                    }
                    $client->owner->password = Hash::make($request->password);
                }
            }

            if (!$client->save()) {
                session()->flash('status', 'error');
                session()->flash('message', 'Sorry! Something went wrong. Please try again');
            }
            session()->flash('status', 'success');
            session()->flash('message', 'Ownership information has been updated.');
            notify(__(''),'success', __('Successful'));
            return back();
        }
        else{
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid Client selected');
            return back();
        }
    }

    public function removeMe(Request $request)
    {
        if (!isset($request->client_id)) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Invalid Organisation.');
        }

        if(auth()->user()->clients()->detach([$request->client_id])) {
            session()->flash('status', 'success');
            session()->flash('message', 'You have exited the organisation');
        }
        return back();
    }

    public function clientApproval(Request $request) {

        $client = Client::find($request->id);

        if (!$client) {
            return response()->json(['status'=>false,'message'=>__('Invalid client information')]);
        }

        if ($client->status) {
            $client->status = 0;
            $client->save();
            return response()->json(['status'=>true,'message'=>__('Client approval has been reversed successfully.')]);
        }
        else {
            $client->status = 1;
            $client->save();
            return response()->json(['status'=>true,'message'=>__('Client has been approved successfully.')]);
        }
    }
}
