<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Middleware\LoadUserPermissions;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\ApplicationApproval;
use App\Models\Benefit;
use App\Models\CaseManagement;
use App\Models\Deployment;
use App\Models\RecordTracker;
use App\Models\User;
use App\Models\UserSaveReport;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class DashboardController extends Controller {

    public function __construct() {

        $this->middleware( ['auth:web'] )->except('home');

    }

    public function index() {

        //dd('Here');

        if (auth()->user()->hasRole('beneficiary')) {
            return redirect()->route('customer.profile');
        }

        if ((auth()->user()->hasRole('owner') && !auth()->user()->hasOrganisation()) ||
            (auth()->user()->onboardingStage && auth()->user()->hasOrganisation())) {
            return  redirect()->route('on_boarding.index');
        }

        $deployment = Deployment::find(getActiveInstance());

        $users = $deployment->users()->count();
        $applicants = $deployment->applicants()->count();

        $total_submitted_records = RecordTracker::select(DB::raw('count(id) as records, status'))
        ->where('instance_id','=',getActiveInstance())->groupBy('status')->pluck('records','status')->toArray();

        $new_records = $in_progress_records = $on_hold_records = $verified_records = $declined_records = 0;
        $new_records_percent = $in_progress_records_percent = $on_hold_records_percent = $verified_records_percent = $declined_records_percent = 0;

        if(isset($total_submitted_records['New'])) $new_records = $total_submitted_records['New'];
        if(isset($total_submitted_records['In Progress'])) $in_progress_records = $total_submitted_records['In Progress'];
        if(isset($total_submitted_records['On Hold'])) $on_hold_records = $total_submitted_records['On Hold'];
        if(isset($total_submitted_records['Declined'])) $declined_records = $total_submitted_records['Declined'];
        if(isset($total_submitted_records['Verified'])) $verified_records = $total_submitted_records['Verified'];
        $total_records = $new_records+$in_progress_records+$on_hold_records+$verified_records+$declined_records;

        //Calculating Percentage
        if($new_records > 0) {
            $new_records_percent = round(($new_records * 100)/$total_records);
        }
        if($in_progress_records > 0) {
            $in_progress_records_percent = round(($in_progress_records * 100)/$total_records);
        }
        if($on_hold_records > 0) {
            $on_hold_records_percent = round(($on_hold_records * 100)/$total_records);
        }
        if($verified_records > 0) {
            $verified_records_percent = round(($verified_records * 100)/$total_records);
        }
        if($declined_records > 0) {
            $declined_records_percent = round(($declined_records * 100)/$total_records);
        }

        $approved_amounts = ApplicationApproval::where('instance_id','=',getActiveInstance())->sum('amount');
        $approved_amount_on = Application::where('instance_id','=',getActiveInstance())->where('status','=','Approved')->count();
        $declined = Application::where('instance_id','=',getActiveInstance())->where('status','=','Declined')->count();

        // $outstanding_records = $new_records+$in_progress_records+$on_hold_records;

        $active_claimers = DB::table('applications')
            ->distinct()
            ->where('instance_id','=',getActiveInstance())
            ->where('status','=','Approved')
//          ->where('is_beneficiary','=',1)
            ->pluck('applicant_number')
            ->toArray();
        $active_claimers_total = count($active_claimers);

        $beneficiaries_by_region = DB::table('applicants')
        ->where('applicants.status','=',1)
        ->whereIn('regID',$active_claimers)
        ->select(DB::raw('count(id) as beneficiaries, place_of_births as region'))
        ->groupBy('place_of_births')
        ->pluck('beneficiaries','region')->toArray();
        $beneficiaries_by_region = json_encode($beneficiaries_by_region,JSON_HEX_TAG);


        // $new_applications = Application::submitted()->count();
        $new_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->where('benefits.archived', '=', 0)
            ->whereNotIn('applications.status', ['Declined','Approved'])
            ->count();
        $active_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->where('applications.archived','=',0)
            ->whereNotIn('applications.status', ['Submitted','Resubmitted','Declined','Approved'])
            ->count();


        /*
        **  Get Year at a Glance Values
        */
        $active_benefits = DB::table('benefits')
            ->leftJoin('applications','benefit_id','=','benefits.id')
             ->select(DB::raw('count(*) as total_applications, benefits.titles as benefit_title'))
            ->where('applications.archived','=',0)
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->groupBy('benefits.titles')
            ->orderBy('benefits.titles','ASC')
            ->pluck('total_applications','benefit_title')->toArray();
        $total_count_apps_benefit = null;
        $apps_benefit_titles = null;

        $count = 1;
        foreach($active_benefits as $key => $value) {
            $total_count_apps_benefit[] = (int)$value;
            $apps_benefit_titles[] = (string)$key;
        }
        $apps_benefit_titles = json_encode($apps_benefit_titles,JSON_HEX_TAG);
        $total_count_apps_benefit = json_encode($total_count_apps_benefit,JSON_HEX_TAG);

        /*
        **  Get Benefit by Gender distribution
        */
        $applications = Application::active()
            ->where('instance_id','=',getActiveInstance())->with(['applicant'])->get();
        // $applications = Application::active()->approved()->with(['applicant'])->get();
        $children = config('settings.age_groups.children');
        $youth = config('settings.age_groups.youth');
        $aged = config('settings.age_groups.aged');
        $childrenTotal = $youthTotal = $agedTotal = $male = $female = 0;
        $age_groups = $age_groups_values = $gender_values = $gender_groups = array();
        $total_applications = $applications->count();

        //Looping through the applications to get applicants information
        foreach ($applications as $application) {
            $applicant = $application->applicant;

            //Checking and collecting age group data of applicants
            $applicant_age = getRawAgeByDate($applicant->dobs);

            if ($applicant_age <= $children[1]) {
                $childrenTotal += 1;
            }
            elseif ($applicant_age <= $youth[1]){
                $youthTotal += 1;
            }
            else {
                $agedTotal += 1;
            }

            //Checking and collecting gender data of applicants
            if ($applicant->genders == 'Male'){
                $male += 1;
            }
            else {
                $female += 1;
            }
        }

        // Collate gender data into a string
        if ($male  > 0){
            $gender_groups[] = 'Male ('.round($male*100/$total_applications).'%)';
            $gender_values[] = $male;
        }
        if ($female > 0){
            $gender_groups[] = 'Female ('.round($female*100/$total_applications).'%)';
            $gender_values[] = $female;
        }
        $gender_groups = json_encode($gender_groups,JSON_HEX_TAG);
        $gender_values = json_encode($gender_values,JSON_HEX_TAG);

        //Collate the age groups data into a string
        if ($childrenTotal > 0) {
            $age_groups[] = 'Children ('.round($childrenTotal*100/$total_applications).'%)';
            $age_groups_values[] = $childrenTotal;
        }
        if ($youthTotal  > 0){
            $age_groups[] = 'Youth ('.round($youthTotal*100/$total_applications).'%)';
            $age_groups_values[] = $youthTotal;
        }
        if ($agedTotal  > 0){
            $age_groups[] = 'Aged ('.round($agedTotal*100/$total_applications).'%)';
            $age_groups_values[] = $agedTotal;
        }
        $age_groups = json_encode($age_groups,JSON_HEX_TAG);
        $age_groups_values = json_encode($age_groups_values,JSON_HEX_TAG);

        $NewAppData = $ApprovedAppData = $PendingAppData = $activeMonths = [];
        $months = [
            1   =>'Jan',
            2   =>'Feb',
            3   =>'Mar',
            4   =>'Apr',
            5   =>'May',
            6   =>'Jun',
            7   =>'Jul',
            8   =>'Aug',
            9   =>'Sep',
            10   =>'Oct',
            11  =>'Nov',
            12  =>'Dec',
        ];
        for ($a = 1; $a <= date('m'); $a++) {
            $NewAppData[$a] = Application::whereIn('status',['Submitted','Resubmitted'])
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $PendingAppData[$a] = Application::whereIn('status',['On Hold','Returned For Correction','Pending Approval'])
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $ApprovedAppData[$a] = Application::where('status','=','Approved')
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $activeMonths[] = $months[$a];
        }

        $NewAppData = json_encode(array_values($NewAppData),JSON_HEX_TAG);
        $PendingAppData = json_encode(array_values($PendingAppData),JSON_HEX_TAG);
        $ApprovedAppData = json_encode(array_values($ApprovedAppData),JSON_HEX_TAG);
        $activeMonths = json_encode(array_values($activeMonths),JSON_HEX_TAG);
        // dd($applications);

        $open_tickets = CaseManagement::where('stage','!=','Close')
            ->where('instance_id','=',getActiveInstance())->count();
        $total_tickets = CaseManagement::where('instance_id','=',getActiveInstance())->count();

        $benefits = getActiveBenefit()->take(6);
        return view( 'dashboard', compact('users','new_applications','benefits','applicants','gender_groups','gender_values',
        'active_applications','total_count_apps_benefit','apps_benefit_titles','age_groups','age_groups_values','beneficiaries_by_region',
        'total_records','new_records','new_records_percent','in_progress_records','in_progress_records_percent','verified_records','declined_records',
        'on_hold_records','on_hold_records_percent','verified_records_percent','declined_records_percent','approved_amounts','ApprovedAppData',
        'activeMonths','NewAppData','PendingAppData','active_claimers_total','approved_amount_on','declined','total_tickets','open_tickets'));
    }

    public function program_dashboard(Request $request)
    {

        if (auth()->user()->hasRole('beneficiary')) {
            return redirect()->route('customer.profile');
        }

        if ((auth()->user()->hasRole('owner') && !auth()->user()->hasOrganisation()) ||
            (auth()->user()->onboardingStage && auth()->user()->hasOrganisation())) {
            return  redirect()->route('on_boarding.index');
        }

        $deployment = Deployment::find(getActiveInstance());

        $users = $deployment->users()->count();
        $applicants = $deployment->applicants()->count();

        $total_submitted_records = RecordTracker::select(DB::raw('count(id) as records, status'))
        ->where('instance_id','=',getActiveInstance())->groupBy('status')->pluck('records','status')->toArray();

        $new_records = $in_progress_records = $on_hold_records = $verified_records = $declined_records = 0;
        $new_records_percent = $in_progress_records_percent = $on_hold_records_percent = $verified_records_percent = $declined_records_percent = 0;

        if(isset($total_submitted_records['New'])) $new_records = $total_submitted_records['New'];
        if(isset($total_submitted_records['In Progress'])) $in_progress_records = $total_submitted_records['In Progress'];
        if(isset($total_submitted_records['On Hold'])) $on_hold_records = $total_submitted_records['On Hold'];
        if(isset($total_submitted_records['Declined'])) $declined_records = $total_submitted_records['Declined'];
        if(isset($total_submitted_records['Verified'])) $verified_records = $total_submitted_records['Verified'];
        $total_records = $new_records+$in_progress_records+$on_hold_records+$verified_records+$declined_records;

        //Calculating Percentage
        if($new_records > 0) {
            $new_records_percent = round(($new_records * 100)/$total_records);
        }
        if($in_progress_records > 0) {
            $in_progress_records_percent = round(($in_progress_records * 100)/$total_records);
        }
        if($on_hold_records > 0) {
            $on_hold_records_percent = round(($on_hold_records * 100)/$total_records);
        }
        if($verified_records > 0) {
            $verified_records_percent = round(($verified_records * 100)/$total_records);
        }
        if($declined_records > 0) {
            $declined_records_percent = round(($declined_records * 100)/$total_records);
        }

        $approved_amounts = ApplicationApproval::where('instance_id','=',getActiveInstance())->sum('amount');
        $approved_amount_on = Application::where('instance_id','=',getActiveInstance())->where('status','=','Approved')->count();
        $declined = Application::where('instance_id','=',getActiveInstance())->where('status','=','Declined')->count();

        // $outstanding_records = $new_records+$in_progress_records+$on_hold_records;

        $active_claimers = DB::table('applications')
            ->distinct()
            ->where('instance_id','=',getActiveInstance())
            ->where('status','=','Approved')
//          ->where('is_beneficiary','=',1)
            ->pluck('applicant_number')
            ->toArray();
        $active_claimers_total = count($active_claimers);

        $beneficiaries_by_region = DB::table('applicants')
        ->where('applicants.status','=',1)
        ->whereIn('regID',$active_claimers)
        ->select(DB::raw('count(id) as beneficiaries, place_of_births as region'))
        ->groupBy('place_of_births')
        ->pluck('beneficiaries','region')->toArray();
        $beneficiaries_by_region = json_encode($beneficiaries_by_region,JSON_HEX_TAG);


        // $new_applications = Application::submitted()->count();
        $new_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->where('benefits.archived', '=', 0)
            ->whereNotIn('applications.status', ['Declined','Approved'])
            ->count();
        $active_applications = DB::table('applications')
            ->join('benefits','benefit_id','=','benefits.id')
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->where('applications.archived','=',0)
            ->whereNotIn('applications.status', ['Submitted','Resubmitted','Declined','Approved'])
            ->count();


        /*
        **  Get Year at a Glance Values
        */
        $active_benefits = DB::table('benefits')
            ->leftJoin('applications','benefit_id','=','benefits.id')
             ->select(DB::raw('count(*) as total_applications, benefits.titles as benefit_title'))
            ->where('applications.archived','=',0)
            ->where('applications.instance_id','=',getActiveInstance())
            ->where('benefits.instance_id','=',getActiveInstance())
            ->groupBy('benefits.titles')
            ->orderBy('benefits.titles','ASC')
            ->pluck('total_applications','benefit_title')->toArray();
        $total_count_apps_benefit = null;
        $apps_benefit_titles = null;

        $count = 1;
        foreach($active_benefits as $key => $value) {
            $total_count_apps_benefit[] = (int)$value;
            $apps_benefit_titles[] = (string)$key;
        }
        $apps_benefit_titles = json_encode($apps_benefit_titles,JSON_HEX_TAG);
        $total_count_apps_benefit = json_encode($total_count_apps_benefit,JSON_HEX_TAG);

        /*
        **  Get Benefit by Gender distribution
        */
        $applications = Application::active()
            ->where('instance_id','=',getActiveInstance())->with(['applicant'])->get();
        // $applications = Application::active()->approved()->with(['applicant'])->get();
        $children = config('settings.age_groups.children');
        $youth = config('settings.age_groups.youth');
        $aged = config('settings.age_groups.aged');
        $childrenTotal = $youthTotal = $agedTotal = $male = $female = 0;
        $age_groups = $age_groups_values = $gender_values = $gender_groups = array();
        $total_applications = $applications->count();

        //Looping through the applications to get applicants information
        foreach ($applications as $application) {
            $applicant = $application->applicant;

            //Checking and collecting age group data of applicants
            $applicant_age = getRawAgeByDate($applicant->dobs);

            if ($applicant_age <= $children[1]) {
                $childrenTotal += 1;
            }
            elseif ($applicant_age <= $youth[1]){
                $youthTotal += 1;
            }
            else {
                $agedTotal += 1;
            }

            //Checking and collecting gender data of applicants
            if ($applicant->genders == 'Male'){
                $male += 1;
            }
            else {
                $female += 1;
            }
        }

        // Collate gender data into a string
        if ($male  > 0){
            $gender_groups[] = 'Male ('.round($male*100/$total_applications).'%)';
            $gender_values[] = $male;
        }
        if ($female > 0){
            $gender_groups[] = 'Female ('.round($female*100/$total_applications).'%)';
            $gender_values[] = $female;
        }
        $gender_groups = json_encode($gender_groups,JSON_HEX_TAG);
        $gender_values = json_encode($gender_values,JSON_HEX_TAG);

        //Collate the age groups data into a string
        if ($childrenTotal > 0) {
            $age_groups[] = 'Children ('.round($childrenTotal*100/$total_applications).'%)';
            $age_groups_values[] = $childrenTotal;
        }
        if ($youthTotal  > 0){
            $age_groups[] = 'Youth ('.round($youthTotal*100/$total_applications).'%)';
            $age_groups_values[] = $youthTotal;
        }
        if ($agedTotal  > 0){
            $age_groups[] = 'Aged ('.round($agedTotal*100/$total_applications).'%)';
            $age_groups_values[] = $agedTotal;
        }
        $age_groups = json_encode($age_groups,JSON_HEX_TAG);
        $age_groups_values = json_encode($age_groups_values,JSON_HEX_TAG);

        $NewAppData = $ApprovedAppData = $PendingAppData = $activeMonths = [];
        $months = [
            1   =>'Jan',
            2   =>'Feb',
            3   =>'Mar',
            4   =>'Apr',
            5   =>'May',
            6   =>'Jun',
            7   =>'Jul',
            8   =>'Aug',
            9   =>'Sep',
            10   =>'Oct',
            11  =>'Nov',
            12  =>'Dec',
        ];
        for ($a = 1; $a <= date('m'); $a++) {
            $NewAppData[$a] = Application::whereIn('status',['Submitted','Resubmitted'])
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $PendingAppData[$a] = Application::whereIn('status',['On Hold','Returned For Correction','Pending Approval'])
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $ApprovedAppData[$a] = Application::where('status','=','Approved')
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',date('Y'))
                ->whereMonth('created_at','=',$a)->count();

            $activeMonths[] = $months[$a];
        }

        $NewAppData = json_encode(array_values($NewAppData),JSON_HEX_TAG);
        $PendingAppData = json_encode(array_values($PendingAppData),JSON_HEX_TAG);
        $ApprovedAppData = json_encode(array_values($ApprovedAppData),JSON_HEX_TAG);
        $activeMonths = json_encode(array_values($activeMonths),JSON_HEX_TAG);
        // dd($applications);

        $open_tickets = CaseManagement::where('stage','!=','Close')
            ->where('instance_id','=',getActiveInstance())->count();
        $total_tickets = CaseManagement::where('instance_id','=',getActiveInstance())->count();

        $benefits = getActiveBenefit()->take(6);
        return view( 'program_dashboard', compact('users','new_applications','benefits','applicants','gender_groups','gender_values',
        'active_applications','total_count_apps_benefit','apps_benefit_titles','age_groups','age_groups_values','beneficiaries_by_region',
        'total_records','new_records','new_records_percent','in_progress_records','in_progress_records_percent','verified_records','declined_records',
        'on_hold_records','on_hold_records_percent','verified_records_percent','declined_records_percent','approved_amounts','ApprovedAppData',
        'activeMonths','NewAppData','PendingAppData','active_claimers_total','approved_amount_on','declined','total_tickets','open_tickets'));

    }
}


