<?php

namespace App\Http\Controllers;

//use App\Classes\ToastNotification;
use App\Models\EmailTemplate;
use App\Models\Role;
use Illuminate\Http\Request;

class EmailsController extends Controller
{
    //
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        $mails = EmailTemplate::all();
        return view('email-template.index', compact('mails'));
    }

    public function create()
    {
        $roles = Role::where('name','<>','webmaster')->get();
        return view('email-template.create', compact('roles'));
    }

    public function edit($id)
    {
        $email = EmailTemplate::find($id);
        $ccs = json_decode($email->cc);
        $v_emails = $emails = null;
        if ($email->receiver_type == 'specific'){
            $emails = json_decode($email->emails);
        }
        elseif ($email->receiver_type == 'specific_in_group'){
            $v_emails = json_decode($email->emails);
        }
        
        $roles = Role::where('name','<>','webmaster')->get();
        return view('email-template.edit', compact('roles','email','ccs','emails','v_emails'));
    }

    public function save(Request $request)
    {
        $this->validate($request,[
            'title' => 'required|string|unique:email_templates',
            'subject' => 'required|string',
            'message' => 'required|string',
            'receiver_type' => 'required|string',
            'g_receivers' => 'required_without:s_receivers|array|min:1',
            's_receivers' => 'required_without:g_receivers|array',
            'cc_list' => 'nullable|array',
        ]);
        
        $email = new EmailTemplate();
        $email->title = $request->title;
        $email->subject = $request->subject;
        $email->message = $request->message;
        $email->receiver_type = $request->receiver_type;

        if ($email->receiver_type == 'group') {
            if (isset($request->g_receivers)) {
                $receivers = implode(',', $request->g_receivers);
                $email->receivers = $receivers;
            }
        }
        else {
            if (isset($request->s_receivers)){
                $email->receivers = json_encode($request->s_receivers);
                $email->emails = json_encode($request->s_receivers);
            }
        }

        if (isset($request->cc_list)){
            $email->cc = json_encode($request->cc_list);
        }

        $email->save();
        if ($email->save()){
            alertNotify('Email Template has been created successfully.',null, 'success');
        }
        else{
            alertNotify('Sorry! Something went wrong! Please try again.',null, 'danger');
        }
        return back();
    }

    public function update(Request $request,$id)
    {
        $this->validate($request,[
            'title' => 'required|string|unique:email_templates,title,'.$id,
            'subject' => 'required|string',
            'message' => 'required|string',
            'receiver_type' => 'required|string',
            'g_receivers' => 'required_without:s_receivers|array|min:1',
            's_receivers' => 'required_without:g_receivers|array',
            'cc_list' => 'nullable|array',
        ]);
        $email = EmailTemplate::find($id);
        $email->title = $request->title;
        $email->subject = $request->subject;
        $email->message = $request->message;
        $email->receiver_type = $request->receiver_type;

        if ($email->receiver_type == 'group') {
            if (isset($request->g_receivers)){
                $receivers = implode(',',$request->g_receivers);
                $email->receivers = $receivers;
            }
        }
        else {
            if (isset($request->s_receivers)){
                $email->receivers = json_encode($request->s_receivers);
                $email->emails = json_encode($request->s_receivers);
            }
        }

        if (isset($request->cc_list)){
            $email->cc = json_encode($request->cc_list);
        }

        $email->save();
        if ($email->save()){
            alertNotify('Email Template has been updated successfully.',null, 'success');
        }
        else{
            alertNotify('Sorry! Something went wrong! Please try again.',null, 'danger');
        }
        return redirect()->route('settings.email_templates');
    }
    
    /**
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy($id)
    {
        if (EmailTemplate::destroy($id)){
            alertNotify('Email Template has been deleted successfully.',null, 'success');
        }
        else{
            alertNotify('Sorry! Something went wrong! Please try again.',null, 'danger');
        }
        return back();
    }
}
