<?php

namespace App\Http\Controllers;

use App\Models\HouseHold;
use App\Models\MemberRecordsComment;
use App\Models\RecordTracker;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;

class MemberRecordsController extends Controller
{
    public function memberRecords(Request $request)
    {
        $records = RecordTracker::orderBy('updated_at','desc');

        if (isset($request->status)){
            $records = $records->where('status','=',$request->status);
        }
        else {
            $records = $records->where('status','!=','Verified');
        }
        if (isset($request->event)){
            $records = $records->where('event','=',$request->event);
        }
        if (isset($request->month)){
            $records = $records->whereMonth('created_at','=',$request->month)->whereYear('created_at','=',date('Y'));
        }
//        if ($request->date_range != null) {
//            $dates = getDateTRange($request->date_range);
//            $records = $records->where('created_at','>=',$dates['from'])->where('created_at','<=',$dates['to']);
//        }

        if (isset($request->refId)){
            $refID = $request->refId;
            $records = $records->where('applicant_number','like',"%$refID%");
        }
        $total_records = RecordTracker::count();
        $new_records = RecordTracker::new()->count();
        $in_progress_records = RecordTracker::progress()->count();
        $on_hold_records = RecordTracker::onHold()->count();
        $verified_records = RecordTracker::verified()->count();
        $declined_records = RecordTracker::declined()->count();

        $records = $records->paginate(100);
        $records->load('requester','applicant');

        return view('member-records.member-records-update',
            compact('records','new_records','total_records','in_progress_records','verified_records','declined_records','on_hold_records'));
    }

    public function getMemberRecord(Request $request)
    {
        $record = RecordTracker::where('id','=',(int)$request->id)->where('applicant_number','=',$request->applicant_number)->first();

        if ($record->table_name == 'applicants') {
            $view = view('member-records.view-record', compact('record'))->render();
        }
        else {
            $view = view('member-records.view-record-array', compact('record'))->render();
        }
        return response()->json(['view'=>$view]);
    }

    public function recordVerification(Request $request)
    {
        $this->validate($request,[
            'comment'    =>  'nullable|string',
        ]);

        if (!in_array($request->submit,config('settings.member_record_status'))) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Action cannot be performed');
            return back();
        }

        $member_record_id = Crypt::decrypt($request->member_record_id);

        $record = RecordTracker::find($member_record_id);
        $record_id = (int)$record->record_id;

        if ($record) {

            if ($record->table_name == 'applicants') {
                if ($request->submit == "Verified") {
                    if($record->field == 'dobs') {
                        $data[$record->field] = defaultDateFormat($record->new_values);
                    }
                    else{
                        $data[$record->field] = $record->new_values;
                    }
                    if ($record->field == 'upload_passport_pic') {
                        $data['avatar'] = $record->new_values;
                    }
                    elseif ($record->field == 'country_of_births') {
                        $data['country_id'] = $record->new_values;
                        $data['country_of_births'] = getCountry($record->new_values,'country_name');
                    }
                    elseif ($record->field == 'state_of_births') {
                        $data['state_id'] = $record->new_values;
                        $data['state_of_births'] = getState($record->new_values,'name');
                    }
                    elseif ($record->field == 'place_of_births') {
                        $data['city_id'] = $record->new_values;
                        $data['place_of_births'] = getCounty($record->new_values,'county');
                    }

                    DB::table('applicants')
                        ->where('id',$record_id)
                        ->update($data);
                }
            }
            else {
                if ($record->event == "Deletion" and $request->submit == 'Verified') {
                    DB::table($record->table_name)->where('id', $record_id)->delete();
                }
                else {

                    $data = $record->new_values;

                    DB::table($record->table_name)
                        ->where('id',$record_id)
                        ->update([
                            'status'    =>  $request->submit
                        ]);
                    if(isset($data['type'])) {
                    if($data['type'] == 137) {
                        if ($data['is_household_head'] == '1') {
                            HouseHold::where('reference_id','=',$data['household_number'])->update([
                                'head'  =>  $record->applicant_number
                            ]);
                        }
                    }
                    }
                }

            }

            $record->status = $request->submit;
            $record->comment = $request->comment;
            if($record->save()) {
                session()->flash('status', 'success');
                session()->flash('message', 'Record has been updated successfully');
                return back();
            }
            else {
                session()->flash('status', 'error');
                session()->flash('message', 'Something went wrong. Please try again.');
                return back();
            }

        }

        session()->flash('status', 'error');
        session()->flash('message', 'Record does not exist');
        return back();
    }

    public function confirmDeleteRequest(Request $request)
    {
        $deleted = DB::table($request->table_name)
            ->where('id','=',$request->record_id)
            ->delete();

        if ($deleted) {
            session()->flash('status', 'success');
            session()->flash('message', 'Record has been deleted successfully');
            return back();
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Something went wrong. Please try again.');
            return back();
        }
    }

    public function uploadDocument($key='attachment',$location='member-records/comments', $validation = 'file|mimes:jpeg,png,jpg,gif,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }
        return $file;
    }

    public function openComments(Request $request)
    {
        $record_id = $request->id;
        $comments = MemberRecordsComment::with(['user'])->where('record_tracker_id','=',$record_id)->get();
        $view = view('member-records.comments', compact('comments','record_id'))->render();

        return response()->json(['view'=>$view]);
    }

    public function addComment(Request $request)
    {
        $request->validate([
            'record_id' =>  'required|integer',
            'comment' =>  'required|string',
            'attachment' =>  'nullable|file',
        ]);

        $attachment = null;
        if (isset($request->attachment)) {
            $attachment = $this->uploadDocument();
        }

        $comments = MemberRecordsComment::create(
            [
                'record_tracker_id' =>  $request->record_id,
                'comment' =>  $request->comment,
                'user_id' =>  auth()->id(),
                'attachment' =>  $attachment,
            ]
        );

        if ($comments) {
            session()->flash('status', 'success');
            session()->flash('message', 'Your comment has be saved');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Please try again');
        }

        return back();
    }
}
