<?php

namespace App\Http\Controllers;

use App\Http\Traits\MembershipTrait;
use App\Http\Traits\ModelTracker;
use App\Models\AccountDetail;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\ApplicationsProcessing;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\Benefit;
use App\Models\ContactDetail;
use App\Models\Customer;
use App\Models\EmploymentDetail;
use App\Models\Entry;
use App\Models\Family;
use App\Models\GeneralNotification;
use App\Models\HealthDetail;
use App\Models\HouseHold;
use App\Models\IdentificationDetail;
use App\Models\Lookup;
use App\Models\MaritalDetail;
use App\Models\Announcement;
use App\Models\Survey;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class MembershipController extends Controller
{
    use ModelTracker, MembershipTrait;


    public function profile()
    {
        $verified_contact = $verified_address = $verified_email = $applications = $eligible_benefits = $benefits =
        $returned_applications = $no_action_applications = null;
        if (auth()->user()->applicant_number != null) {
            $verified_contact = auth()->user()->applicant?->contactDetails->where('status','Verified')
                ->where('default',1)
                ->where('type_name','Mobile Number')
                ->pluck('mobile')
                ->first();
            $verified_email = auth()->user()->applicant?->contactDetails->where('status','Verified')
                ->where('default',1)
                ->where('type_name','Email')
                ->pluck('email')
                ->first();
            $verified_address = auth()->user()->applicant?->contactDetails->where('status','Verified')
                ->where('default',1)
                ->where('type_name','Residential Address')
                ->pluck('ResidentialAddress')
                ->first();
            $applications = Application::with('applicationProcessings','benefit','lookup')->active()->where('applicant_number',auth()->user()->applicant_number)->get();

            $eligible_benefits = [];
            if(auth()->user()->applicant) $eligible_benefits = applicant_eligible_benefits(auth()->user()->applicant->regID);

            $benefits = Benefit::with(['deployment'])->whereIn('id',$eligible_benefits)->whereNotNull('form_id')->orderBy('created_at','desc')->take(5)->get();

            $returned_applications = $applications->where('status','=','Returned For Correction')->pluck('id','benefit_id')->toArray();
            $no_action_applications = $applications->where('status','!=','Returned For Correction')->pluck('id','benefit_id')->toArray();
        }

        $notification = Announcement::published()->orderBy('updated_at','desc')->first();
//        dd($notification);

        return view('customers.profile', compact('verified_contact','verified_email',
            'verified_address','eligible_benefits','benefits','applications','notification','returned_applications','no_action_applications'));
    }

    public function eligibleBenefits(Request $request) {
        $applications = Application::with('applicationProcessings','benefit','lookup')->active()->where('applicant_number',auth()->user()->applicant_number)->get();
        $eligible_benefits = [];
        if(auth()->user()->applicant) $eligible_benefits = applicant_eligible_benefits(auth()->user()->applicant->regID);
        $returned_applications = $applications->where('status','=','Returned For Correction')->pluck('id','benefit_id')->toArray();
        $no_action_applications = $applications->where('status','!=','Returned For Correction')->pluck('id','benefit_id')->toArray();
        $benefits = Benefit::with(['deployment'])->whereIn('id',$eligible_benefits)->whereNotNull('form_id')->orderBy('created_at','desc');
        if ($request->country) {
            $benefits = $benefits->where('country_id',$request->country)->orWhereNull('country_id');
        }
        if ($request->financial) {
            $benefits = $benefits->where('financial',$request->financial);
        }
        $benefits = $benefits->paginate(50);
        return view('customers.eligible-benefits', compact('applications','benefits','returned_applications','no_action_applications'));
    }

    public function connectToApplication(Request $request)
    {
        $request->validate([
            'applicant_number' => 'required|string'
        ]);

        $applicant = Applicant::where('regID',$request->applicant_number)->first();

        if ($applicant) {
            if ($applicant->assigned == 1) {
                $message = 'Sorry! the beneficiary account has already been assigned. Please contact support for assistance.';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }

            auth()->user()->pending_application = $request->applicant_number;
            auth()->user()->save();

            $code = verificationCode();

            $applicant->assigned_code = $code;
            $applicant->save();
            $sms = setAppDefault('app_project_name').": Use the verification code - $code to reset your password. Thank you.";
            sendSMS(auth()->user()->phone,$sms);

            $message = 'Successful! Please enter the 4-digit code sent to your phone to verify.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = 'Sorry! the beneficiary account does not exist.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
    }

    public function verifyApplication(Request $request)
    {

        if (isset($request->submit)) {
            $request->validate([
                'code' => 'required|string|min:4|max:4'
            ]);
            $applicant = Applicant::where('regID',auth()->user()->pending_application)->where('assigned_code',$request->code)->first();

            if ($applicant) {
                $applicant->assigned_code = null;
                $applicant->assigned = 1;
                $applicant->save();

                auth()->user()->pending_application = null;
                auth()->user()->applicant_number = $applicant->regID;

                if(auth()->user()->save()) {
                    User::where('pending_application',$applicant->regID)->where('id','<>',auth()->id())->update([
                        'pending_application' => NULL
                    ]);
                }

                $message = 'Account has been attached successfully.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                return back();
            }
            else {
                $message = 'The code is invalid.';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }
        }

        if (isset($request->detach)) {
            auth()->user()->pending_application = null;
            auth()->user()->save();
            $message = 'The Beneficiary Account has been detached.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        return back();
    }

    public function viewApplication($application_id)
    {
        $verified_contact = auth()->user()->applicant->contactDetails->where('status','Verified')
            ->where('default',1)
            ->where('type_name','Mobile Number')
            ->pluck('mobile')
            ->first();
        $verified_email = auth()->user()->applicant->contactDetails->where('status','Verified')
            ->where('default',1)
            ->where('type_name','Email')
            ->pluck('email')
            ->first();
        $verified_address = auth()->user()->applicant->contactDetails->where('status','Verified')
            ->where('default',1)
            ->where('type_name','Residential Address')
            ->pluck('ResidentialAddress')
            ->first();


        //Get the Benefit Registration data
        $application = Application::with(['benefit', 'addedBy', 'applicant'])->where('id', Crypt::decrypt($application_id))->firstOrFail();
        $applicant = Applicant::where('regID', '=', $application->applicant_number)->first();
        updateNotification(Crypt::decrypt($application_id),$application->getTable());

        $used_documents = $applicant->documents->whereIn('type', $application->documentations);

        $channel = $this->applicantChannelDetails($application->bank_account_id);
        $notification = Announcement::published()->orderBy('updated_at','desc')->first();
        $applications = Application::with('applicationProcessings','benefit','lookup')->active()->where('applicant_number',auth()->user()->applicant_number)->get();


        return view('customers.view-application',
            compact('verified_contact','verified_email','notification','applications',
                'verified_address','applicant', 'application', 'channel', 'used_documents'));
    }

    public function applicantChannelDetails($channelId)
    {
        $accountDetail = AccountDetail::find($channelId);
        if ($accountDetail) {
            return  view('customers.bank-account-info', compact('accountDetail'))->render();
        }
        else {
            return '<h5 class="text-center mt-5 text-danger">No Record Found</h5>';
        }
    }

    public function account()
    {
        $page_title = __('New Beneficiary Enrolment');
        $applicant = null;
        $applicant_docs = null;
        $pending_approval = false;
        $is_new = true;
        if (isset(auth()->user()->applicant)) {
            $page_title = __('Edit Account');
            $applicant = auth()->user()->applicant;
            $applicant_docs = $applicant->documents;
            $is_new = false;

            if ($applicant->history->where('status','=',0)->where('approved','=',0)->count() > 0) {
                $pending_approval = true;
            }
        }

        $islands = Lookup::where('lookup_code_id','=', config('settings.island_code'))->pluck('fullName','short_code')->toArray();
        $countries = DB::table('countries')->orderBy('country_name')->get();
        $place_of_birth = $islands;
        $nationalities = null;
        return view('customers.account', compact('page_title','is_new','place_of_birth','nationalities','applicant','applicant_docs','countries'));
    }

    /**
     * Function for upload image
     *
     * @param  string  $folder
     * @param  string  $key
     * @param  string  $validation
     *
     * @return false|string|null
     */
    public function uploadProfile($key = 'app_logo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = 'applicant/profile/'.auth()->id();
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
            $file = 'storage/'.$file;
        }

        return $file;
    }
    public function uploadFile($key,$location, $validation = 'file|mimes:jpeg,png,jpg,gif,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }
        return $file;
    }
    public function uploadMultipleFiles($files,$location)
    {
        $file = [];
        foreach($files as $evidence) {
            $name = $evidence->hashName();
            $file[] = Storage::disk('local')->putFileAs($location, $evidence,$name);
        }
        return $file;
    }

    /**
     * Save Customer Data
     * @param Request $request
     *
     * @return string
     */

    public function saveCustomerRegistration(Request $request)
    {
        if($this->createMemberShip($request)) {
            //return redirect()->route('customer.account');
            return redirect()->route('applicant.getApplicantInfo');
        }
        return back();
    }

    /**
     * Update Applicant Record
     * @param Request $request
     *
     * @return string
     */
    public function updateCustomerRecord(Request $request)
    {
        //dd(get_region_by_name('Taita–Taveta', 'id'));
        if (isset($request->request_delete)){
            if ($request->request_delete != "true"){
                if (ContactDetail::where('mobile','=',$request->mobile)->count() > 0) {
                    $message = 'Sorry! This mobile number is already in use.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back();
                }

                if (ContactDetail::where('email','=',$request->email)->count() > 0) {
                    $message = 'Sorry! This email is already in use.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back();
                }
            }
        }

        return $this->updateMembershipRecord($request);
    }

    public function verify()
    {
        return view('auth.customer-verify');
    }

    public function verifyCode(Request $request)
    {
        $this->validate($request, [
            'verification_code' => 'required|string|min:4|max:4',
        ]);

        if (Str::upper(auth()->user()->verification_code) == Str::upper($request->verification_code)) {
            auth()->user()->verification_code = null;
            auth()->user()->account_verified_at = now();
            auth()->user()->status = 1;
            auth()->user()->save();
            $message = 'Your account has been verified.';
            session()->flash('status', 'success');
        }
        else {
            $message = "Sorry! Invalid Code.";
            session()->flash('status', 'error');
        }
        session()->flash('message', $message);

        return back();
    }

    public function resendVerifyCode()
    {
        $code = strtoupper(Str::random(4));
        if (auth()->user()->account_verified_at == null) {
            auth()->user()->verification_code = $code;
            auth()->user()->save();
            $sms = setAppDefault('app_project_name') . ": Use this verification code - $code to activate your account. Thank you.";
            sendSMS(auth()->user()->phone, $sms);
            $message = 'A new verification code has been sent to your phone number.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
        } else {
            $message = "Your account as already been verified.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }

        return back();
    }

    /**
     * New Benefit Registrations.
     * @param Request $request
     * @param null $form_id
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\RedirectResponse
     */
    public function benefitEnrolment($benefit_id)
    {
        $application_form =  null;
        $applicant_regID = auth()->user();

        $benefit = Benefit::with('form')->where('id',Crypt::decrypt($benefit_id))->first();
        $benefit_name =  null;

        $application_form_view =  'general-form';

        if($benefit) {
            $benefit_id = $benefit->id;
            $benefit_name = $benefit->titles;
            $application_form_view = $benefit->form->view_file;
        }
        else {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        if (is_null($application_form_view)) {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        $header = 'General Application Form';

        return view("benefit-registration.application-forms.$application_form_view",
            compact('header','application_form','benefit_id','benefit_name','applicant_regID'));
    }

    /**
     * Edit Benefit Registrations.
     * @param Request $request
     * @param null $form_id
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\RedirectResponse
     */
    public function editBenefitEnrolment($application_id)
    {
        $application = Application::findOrFail(Crypt::decrypt($application_id));
        $application_form =  null;
        $applicant_regID = auth()->user();

        $benefit = Benefit::with('form')->where('id','=',$application->benefit_id)->first();
        $benefit_name =  null;

        $return_message = null;
        if ($application->status == 'Returned For Correction'){
            $app = $application->latestReturnForCorrection();
            $return_message = $app->remarks;
        }

        $application_form_view =  'edit-general-form';

        if($benefit) {
            $benefit_id = $benefit->id;
            $benefit_name = $benefit->titles;
            $application_form_view = 'edit-'.$benefit->form->view_file;
        }
        else {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        if (is_null($application_form_view)) {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        $header = 'General Application Form';

        return view("benefit-registration.application-forms.$application_form_view",
            compact('header','return_message','application_form','benefit_id','application','benefit_name','applicant_regID'));
    }

    //Edit corresponding code in ApiController
    public function validateApplicant(Request $request)
    {
        if (auth()->user()->applicant_number != $request->applicant) {
            return response()->json(['error'=>'Applicant account is invalid','message'=>'Sorry','status'=>0]);
        }
        $eligible_data = isApplicantEligible(auth()->user()->applicant_number,$request->benefit);
        if(!$eligible_data['status']) {
            return response()->json(['error'=>'Applicant is not eligible for this benefit claim','message'=>$eligible_data['message'],'status'=>0]);
        }

        $application_bank_account_id = null;
        if (isset($request->application_bank_account_id)) $application_bank_account_id = $request->application_bank_account_id;

        if (!is_null(auth()->user()->applicant_number)) {
            $applicant = Applicant::find(auth()->user()->applicant->id);
            $benefit = Benefit::find($request->benefit);
            $documents = $benefit->supporting_documents;
            $applicant_documents = auth()->user()->applicant->documents->pluck('type','id')->toArray();

            $documents = view('benefit-registration.applications.documents-info', compact('applicant_documents','documents'))->render();
            $view = view('benefit-registration.applications.basic-info', compact('benefit','applicant','application_bank_account_id'))->render();
            return response()->json(['applicant'=>auth()->user()->applicant->fullname,
                'id'=>cryptThis(auth()->user()->applicant->id),
                'view'=>$view,  'documents'   => $documents ]);
        }
        else {
            return response()->json(['error'=>'No Record Found','status'=>0]);
        }
    }

    public function applicantAccountDetails(Request $request)
    {
        $accountDetail = AccountDetail::find($request->num);
        if ($accountDetail) {
            $view = view('benefit-registration.applications.bank-account-info', compact('accountDetail'))->render();
            return response()->json($view);
        }
        else {
            return response()->json('<h5 class="text-center mt-5 text-danger">No Record Found</h5>');
        }
    }

    /**
     * @param Request $request
     * @param $benefit_id
     * @param $applicant_number
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveEnrolment(Request $request,$benefit_id,$applicant_number=null)
    {
        $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        $receivers = benefitRespondents($benefit->respondent);
        $receiver_ids = $receivers->pluck('id')->toArray();
//        dd($receiver_ids);

        if ($request->comfirmation != 1) {
            $message = 'Sorry! You must agree and confirm the terms of this application to proceed.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        if (is_null($benefit)) {
            $message = 'Sorry! Benefit does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        $docs = $request->documentations_existing;

        $applicant = Applicant::where('regID','=',auth()->user()->applicant_number)->first();
        if (is_null($applicant)) {
            $message = 'Sorry! Applicant does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        try{
            if (!is_null($request->documentations_new)) {
                foreach($request->documentations_new as $key=>$document){
                    $doc = Str::snake($document);
                    $doc_name = "upload_$doc";
                    $file_path = $this->uploadFile($doc_name,'/applicants/'.$applicant->regID.'/application');
                    $docs[] = $document;
                    addApplicantDocument($applicant->id,$file_path,$document);
                }
            }

            $app = Application::create([
                'instance_id' => $benefit->instance_id,
                'form_id' => $benefit->form_id,
                'benefit_id' => $benefit->id,
                'applicant_number' => auth()->user()->applicant_number,
                'applicationable_id' => Auth::id(),
                'applicationable_type' => 'App/Models/User',
                'modified_by' => Auth::id(),
                'type' => $benefit->titles,
                'bank_account_id' => $request->bank_account,
                'reporter_id_number' => $request->reporter_id_number,
                'reporter_surname' => $request->reporter_surname,
                'reporter_first_name' => $request->reporter_first_name,
                'reporter_contact' => $request->reporter_contact,
                'reporter_id_type' => $request->reporter_id_type,
                'reporter_relationship' => $request->reporter_relationship,
                'user_type' => 'applicant',
                'documentations' => $docs,
                'status' => 'Submitted',
            ]);

            $access_type = 'Portal';
            if ($access_type != 'Portal') {
                $access_type = 'Mobile App';
            }

            if ($app) {
                ApplicationsProcessing::create([
                    'instance_id' => $benefit->instance_id,
                    'application_id'    =>  $app->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  'Submitted',
                    'access_type'            =>  $access_type,
                    'stage'             =>  'New Application',
                    'remarks'             =>  null
                ]);
            }

            GeneralNotification::create([
                'user_id'   =>  auth()->id(),
                'receivers'   =>  $receiver_ids,
                'subject_id'   =>  $app->id,
                'table'   =>  $app->getModel()->getTable(),
                'instance_id'   =>  $app->instance_id,
                'message'   =>  'A new '.$benefit->titles.' application as been submitted by #'.$app->applicant_number,
                'route'   =>  route('benefit_registration.view',[cryptThis($app->id)]),
            ]);

            $message = "Application has been submitted.";
            session()->flash('status', 'success');
            session()->flash('message', $message);

            return redirect()->route('customer.profile');
        }
        catch (\Exception $exception){
            $message = 'Sorry! Submission failed. Try again. '.$exception->getMessage();
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

    }
    /**
     * @param Request $request
     * @param $benefit_id
     * @param $applicant_number
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function updateEnrolment(Request $request)
    {
        $application = Application::findOrFail(decrypt($request->application_id));

        $benefit = Benefit::find($application->benefit_id);
        if ($request->comfirmation != 1) {
            $message = 'Sorry! You must agree and confirm the terms of this application to proceed.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        if (is_null($benefit)) {
            $message = 'Sorry! Benefit does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        $docs = $request->documentations_existing;

        $applicant = Applicant::where('regID','=',auth()->user()->applicant_number)->first();
        if (is_null($applicant)) {
            $message = 'Sorry! Applicant does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        try{
            if (!is_null($request->documentations_new)) {
                foreach($request->documentations_new as $key=>$document){
                    $doc = Str::snake($document);
                    $doc_name = "upload_$doc";
                    $file_path = $this->uploadFile($doc_name,'/applicants/'.$applicant->regID.'/application');
                    $docs[] = $document;
                    addApplicantDocument($applicant->id,$file_path,$document);
                }
            }

            $app = Application::updateOrCreate([
                'id'    =>  Crypt::decrypt($request->application_id),
            ],[
                'form_id' => $benefit->form_id,
                'benefit_id' => $benefit->id,
                'applicant_number' => auth()->user()->applicant_number,
                'applicationable_id' => Auth::id(),
                'applicationable_type' => 'App/Models/User',
                'modified_by' => Auth::id(),
                'type' => $benefit->titles,
                'bank_account_id' => $request->bank_account,
                'reporter_id_number' => $request->reporter_id_number,
                'reporter_surname' => $request->reporter_surname,
                'reporter_first_name' => $request->reporter_first_name,
                'reporter_contact' => $request->reporter_contact,
                'reporter_id_type' => $request->reporter_id_type,
                'reporter_relationship' => $request->reporter_relationship,
                'user_type' => 'applicant',
                'documentations' => $docs,
                'status' => 'Resubmitted',
            ]);

            $access_type = 'Portal';
            if ($access_type != 'Portal') {
                $access_type = 'Mobile App';
            }

            if ($app) {
                ApplicationsProcessing::create([
                    'application_id'    =>  $app->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  'Resubmitted',
                    'access_type'            =>  $access_type,
                    'stage'             =>  'Correction',
                    'remarks'             =>  null
                ]);
            }


            $message = "Application has been submitted.";
            session()->flash('status', 'success');
            session()->flash('message', $message);

            return redirect()->route('customer.profile');
        }
        catch (\Exception $exception){
            $message = 'Sorry! Submission failed. Try again. '.$exception->getMessage();
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

    }

    public function mySurveys()
    {

        $surveys = Survey::with(['surveyEntries' => function ($query) {
                $query->where('participant_id','=', auth()->id())->orderBy('created_at','desc');
            }])
                ->whereDate('start_date','<=',now())
                ->where('mode','=','self')
                ->where('self_respondents','=','all-users')
                ->orWhere('self_respondents','=','applicants')->paginate(100);

        return view('customers.my-surveys', compact('surveys'));
    }

    public function viewSurveyEntry(Entry $entry)
    {
        return view('customers.view-survey-entry', compact('entry'));
    }

    public function newEntry(Survey $survey)
    {
        $submit = true;

        if (strtotime($survey->end_date) < strtotime(date('Y-m-d'))) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! The survey is overdue.');
            return back();
        }

        if (strtotime($survey->start_date) > strtotime(date('Y-m-d'))) {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Invalid Survey.');
            return back();
        }

        return view('customers.entry', compact('submit','survey'));
    }

    public function saveEntry(Survey $survey,Request $request)
    {
        $answers = $this->validate($request, $survey->form->rules);

        $entry = (new Entry)->for($survey->form)->onSurvey($survey)->by(auth()->user())->fromArray($answers)->push();

        if($entry){
            $message = "Entry has been saved";
            $status = 'success';
        }
        else {
            $message = "Something went wrong. Try again.";
            $status = 'error';
        }
        session()->flash('status', $status);
        session()->flash('message', $message);
        return redirect()->route('customer.mySurveys');
    }

    public function printPreview()
    {
        return view('benefit-registration.application-forms.general-form-print');
    }
}
