<?php

namespace App\Http\Controllers;

use App\Mail\OnBoarding;
use App\Models\Client;
use App\Models\Deployment;
use App\Models\DeploymentsLog;
use App\Models\Module;
use App\Models\OnboardingStage;
use App\Models\Plan;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;

class OnBoardingController extends Controller
{
    public function __construct()
    {
        return $this->middleware(['auth','verified']);
    }

    public function index()
    {
        $stage = auth()->user()->onboardingStage?->stage;

        if ($stage) {
            if ($stage == 'personal_details') {
                return redirect()->route('on_boarding.personalDetails');
            }
            elseif ($stage == 'project_setup') {
                return redirect()->route('on_boarding.setupProject');
            }
            elseif ($stage == 'booking_a_meeting') {
                return redirect()->route('on_boarding.setupMeeting');
            }
        }

        return redirect()->route('on_boarding.client');
    }

    public function organizationSetup()
    {
        return view('on-boarding.sign-up');
    }
    public function personalDetails()
    {
        return view('on-boarding.owner-details');
    }
    public function setupProject()
    {
        return view('on-boarding.project-setup');
    }

    public function setupMeeting()
    {
        $size = 10;
        $offset_size = 1;
        return view('on-boarding.meeting-setup', compact('offset_size','size'));
    }

    public function selectedMeetingDate()
    {
        OnboardingStage::where('user_id','=',auth()->id())->delete();
        return redirect()->route('dashboard');
    }

    public function create(Request $request)
    {
        $this->validate($request, [
            'business_name' => 'required|string',
            'country' => 'required|integer',
            'type' => 'required|string',
            'size' => 'required|string',
        ]);

        $client = auth()->user()->hasOrganisation();
        if (!$client) {
            $client = new Client();
        }

        $client->name =  $request->business_name;
        $client->country_id =  $request->country;
        $client->field =  $request->type;
        $client->size =  $request->size;
        $client->admin =  auth()->id();
        $client->status =  0;
        $client->created_by =  auth()->id();
        if ($client->save()) {
            auth()->user()->clients()->attach([$client->id]);

            OnboardingStage::UpdateOrCreate([
                'user_id'   =>  auth()->id()
            ],[
                'stage' =>  'personal_details'
            ]);

            session()->flash('status', 'success');
            session()->flash('message', 'Organization information saved');
            return redirect()->route('on_boarding.personalDetails');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry please try again');
            return back();
        }
    }
    public function update_profile(Request $request)
    {

        $this->validate($request, [
            'firstname' => 'required|string',
            'surname' => 'required|string',
            'country' => 'required|integer',
            'call_code' => 'required|integer',
            'phone_number' => 'required|string|min:9|max:9',
        ]);

        auth()->user()->name =  $request->surname;
        auth()->user()->firstname =  $request->firstname;
        auth()->user()->country_id =  $request->country;
        auth()->user()->call_code =  $request->call_code;
        auth()->user()->phone =  $request->phone_number;
        if (auth()->user()->save()) {
            OnboardingStage::UpdateOrCreate([
                'user_id'   =>  auth()->id()
            ],[
                'stage' =>  'project_setup'
            ]);
            session()->flash('status', 'success');
            session()->flash('message', 'Personal details saved');
            return redirect()->route('on_boarding.setupProject');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry please try again');
            return back();
        }
    }
    public function updateProject(Request $request)
    {
        $this->validate($request, [
            'app_project_module' => 'required|string',
            'project_name' => 'required|string|min:3',
            'country' => 'required|integer',
            'administrator_email' => 'required|email',
        ]);

        $deployment = auth()->user()->hasOrganisation()->deployment;
        if(is_null($deployment)) {
            $deployment = new Deployment();
        }

        $deployment->name = $request->project_name;
        $deployment->module = $request->app_project_module;
        $deployment->email = $request->administrator_email;
        $deployment->deployment_date = date('Y-m-d');
        $deployment->code = deploymentCode();
        $deployment->status = 'Deployed';
        $deployment->instance = 'Online';
        $deployment->client_id = auth()->user()->hasOrganisation()->id;
        $deployment->country_id = $request->country;
        $deployment->country_name =  getCountry($deployment->country,'country_name');
        $deployment->abbreviation = strtoupper(substr($request->project_name, 0, 3));
        if ($deployment->save()) {

            $plan = Plan::with(['features'])->whereSlug('free')->first();
            $deployment_date = Carbon::now();
            $subscribed = $deployment->newPlanSubscription($deployment->code, $plan,$deployment_date);

            $deployment->expiry = $subscribed->ends_at;
            $deployment->package = $plan->name;
            $deployment->save();

            $featuresMenu = $features = null;
            if (isset($plan->features)) {
                $featuresMenu = $plan->features->setVisible(['name','value','icon','route','description'])->toArray();
                $features = $plan->features->pluck('name')->toArray();
            }
            Module::updateOrCreate([
                'instance_id'   =>  $deployment->id
            ],[
                'active'   =>  1,
                'modules'   =>  $features,
                'menus'   =>  $featuresMenu,
                'applicant_code'   =>  null,
                'instance'   =>  'Online',
            ]);

            $deploymentLog = new DeploymentsLog();
            $deploymentLog->status =  $deployment->status;
            $deploymentLog->instance =  $deployment->instance;
            $deploymentLog->expiry =  $deployment->expiry;
            $deploymentLog->amount =  $deployment->amount;
            $deploymentLog->country_id =  $deployment->country_id;
            $deploymentLog->client_id =  $deployment->client_id;
            $deploymentLog->package =  $deployment->package;
            $deploymentLog->subscription_interval =  $deployment->subscription_interval;
            $deploymentLog->deployment_id =  $deployment->id;
            $deploymentLog->created_by =  auth()->id();
            $deploymentLog->save();

            auth()->user()->touring_areas = config('settings.tour-pages');
            auth()->user()->save();
            OnboardingStage::UpdateOrCreate([
                'user_id'   =>  auth()->id()
            ],[
                'stage' =>  'booking_a_meeting'
            ]);

            auth()->user()->deployments()->attach([$deployment->id]);

            //Re-assign the instance to the user role.
            DB::table('role_user')->where('role_id','=',env('OWNER_ROLE'))
                ->where('user_id','=',auth()->id())
                ->update(['instance_id'=>$deployment->id]);

            auth()->user()->last_instance_opened = $deployment->id;
            auth()->user()->save();
            session()->flash('status', 'success');
            session()->flash('message', 'Project details saved');
            return redirect()->route('on_boarding.setupMeeting');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry please try again');
            return back();
        }
    }

    public function verifyAccount(Request $request, $user_id)
    {
        $user = User::find($user_id);

        if (!$user) {
            abort(404);
        }

        if($user->remember_token != $request->token) {
            abort(404);
        }

        if($request->expires_on < strtotime(now())) {
            session()->flash('status', 'error');
            session()->flash('message', 'The verification link has expired.');
            return redirect()->route('on_boarding.index');
        }

        $user->remember_token = null;
        $user->account_verified_at = null;
        $user->email_verified_at = null;
        if($user->save()) {
            session()->flash('status', 'success');
            session()->flash('message', 'Your account has been verified. Please proceed to login.');
            return redirect()->route('login');
        }
    }

    public function locationPopoverContent()
    {
        $location = auth()->user()->has_toured;
        if ($location == 0) {
            $view = view('on-boarding.popovers',compact('location'))->render();
            return response()->json(['locationView'=>$view]);
        }
        else {
            $view = view('on-boarding.popovers',compact('location'))->render();
            return response()->json(['locationView'=>$view]);
        }
    }

    function skipTour(Request $request) 
    {
        if($request->page == 'end'){
            auth()->user()->has_toured = true;
            auth()->user()->touring_areas = null;
        }
        else {
            $areas_left = findReplaceItem($request->page,auth()->user()->touring_areas);
            auth()->user()->touring_areas = $areas_left;

            if(is_null($areas_left) || sizeof($areas_left) < 1) {
                auth()->user()->has_toured = true;
                auth()->user()->touring_areas = null;
            }
        }
        
        auth()->user()->save();
    }
}
