<?php

namespace App\Http\Controllers;

use App\Models\Application;
use App\Models\ApplicationApproval;
use App\Models\MasterPaymentVoucher;
use App\Models\PaymentVoucher;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class PaymentController extends Controller
{

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    public function paymentVouchers(Request $request)
    {
        $master_list = MasterPaymentVoucher::orderBy('period','asc');
        $year = date('Y');

        if(isset($request->year)) {
            $year = $request->year;
            $master_list = $master_list->whereYear('period','=',$year);
        }
        else {
            $master_list = $master_list->whereYear('period','=',$year);
        }

        if(isset($request->status)) {
            $master_list = $master_list->where('status','=',$request->status);
        }

        $master_list = $master_list->get();
        $months = config('settings.months');
        $available_master_list = [];
        foreach ($master_list as $list) {
            $month = date('F',strtotime($list->period));
            $available_master_list[$month]['id'] = $list->id;
            $available_master_list[$month]['voucher_number'] = $list->voucher_number;
            $available_master_list[$month]['voucher_date'] = date('d M Y',strtotime($list->voucher_date));
            $available_master_list[$month]['beneficiaries'] = $list->paymentVouchers->count();
            $available_master_list[$month]['gross_pay'] = getCurrency().$list->gross_pay;
            $available_master_list[$month]['tax'] = $list->tax."%";
            $available_master_list[$month]['netable_pay'] = getCurrency().$list->netable_pay;
            $available_master_list[$month]['status'] = $list->status;
        }

        return view('payments.payment-vouchers', compact('available_master_list','months','year'));
    }

    public function loadVoucherForm(Request $request)
    {
        if (!isset($request->month)) {
            return response()->json(['formView'=>'<div class="text-center p-5"><h1 class="text-muted"> Invalid month selected</h1></div>']);
        }
        $month = $request->month;

        $benefit_aggregate = DB::table('application_approvals')
            ->join('applications','applications.id','=','application_approvals.application_id')
            ->join('benefits','applications.benefit_id','=','benefits.id')
            ->selectRaw('count(application_approvals.id) as number_of_claims,sum(application_approvals.amount) as claims_sum, benefits.titles as title')
            ->whereMonth('application_approvals.created_at','=',$month)
            ->whereYear('application_approvals.created_at','=',date('Y'))
            ->groupBy('benefits.titles')
            ->get();

        $benefit_aggregate_titles = json_encode($benefit_aggregate->pluck('title')->toArray(),JSON_HEX_TAG);
        $benefit_aggregate_claims = json_encode($benefit_aggregate->pluck('number_of_claims')->toArray(),JSON_HEX_TAG);

        $approved_claims = ApplicationApproval::whereMonth('created_at','=',$month)
            ->whereYear('created_at','=',date('Y'))
            ->count();
        $approved_claims_amount = ApplicationApproval::whereMonth('created_at','=',$month)
            ->whereYear('created_at','=',date('Y'))
            ->sum('amount');

        $view = view('payments.generate-voucher-form', compact('approved_claims','month',
            'benefit_aggregate','approved_claims_amount','benefit_aggregate_titles','benefit_aggregate_claims'))->render();
        return response()->json(['formView'=>$view]);
    }

    public function generateVoucher(Request $request)
    {
        if (!isset($request->month)) {
            session()->flash('message','Incorrect month selected');
            session()->flash('status','error');
            return back();
        }

        $tax = 0;
        $payment_days_interval = 14;
        $month = $request->month;

        $gross_pay = DB::table('application_approvals')
            ->selectRaw('sum(amount) as claims_sum')
            ->whereMonth('created_at','=',$month)
            ->whereYear('created_at','=',date('Y'))->first()->claims_sum;

        $netable_pay = ((100-$tax)/100)*$gross_pay;

        $payment_date = date('Y-m-d H:i:s', strtotime("+$payment_days_interval days"));
        $period = date('Y').'-'.$month.'-01';

        $already_exists = MasterPaymentVoucher::whereMonth('period','=',$month)->whereYear('period','=',date('Y'))->first();
        if (!is_null($already_exists)) {
            session()->flash('message','Voucher has already been generated for this period');
            session()->flash('status','error');
            return back();
        }
        $get_applications = ApplicationApproval::with(['application'])
            ->whereMonth('created_at','=',$month)
            ->whereYear('created_at','=',date('Y'))
            ->get();

        $master_voucher = new MasterPaymentVoucher();
        $master_voucher->voucher_date = date('Y-m-d H:i:s');
        $master_voucher->payment_date = $payment_date;
        $master_voucher->period = $period;
        $master_voucher->gross_pay = $gross_pay;
        $master_voucher->tax = $tax;
        $master_voucher->netable_pay = $netable_pay;
        $master_voucher->status = config('settings.payment_voucher_start_status');
        $master_voucher->created_by = auth()->id();

        if ($master_voucher->save()){
            $master_voucher->voucher_number = 'PV-'.date('Y').$master_voucher->id;
            $master_voucher->save();

            foreach ($get_applications as $get_application) {
                $benefit_tax = 0;
                PaymentVoucher::updateOrCreate([
                    'applicant_ref_no'  =>  $get_application->application->applicant_number,
                    'voucher_id'  =>  $master_voucher->id,
                    'benefit_id'  =>  $get_application->application->benefit_id,
                    'period'  =>  $master_voucher->period,
                ],[
                    'pay_type'  =>  null,
                    'gross_pay'  =>  $get_application->amount,
                    'tax'  =>  $benefit_tax,
                    'netable_pay'  =>  ((100-$tax)/100)*$get_application->amount,
                    'status'  =>  config('settings.payment_voucher_start_status'),
                    'created_by'  =>  auth()->id(),
                ]);
            }

            session()->flash('message','Voucher has been generated');
            session()->flash('status','success');
        }
        else {

            session()->flash('message','Something went wrong. Please try again.');
            session()->flash('status','error');
        }

        return back();
    }

    public function paymentBatchVoucher($master_payment_voucher_id)
    {
        $master = MasterPaymentVoucher::find($master_payment_voucher_id);
        $voucher = PaymentVoucher::where('voucher_id','=',$master_payment_voucher_id)->get();
        return view('payments.payment-batch-voucher', compact('voucher','master'));
    }

    public function paymentBatch($master_payment_voucher_id)
    {
        $voucher_list = PaymentVoucher::where('voucher_id','=',$master_payment_voucher_id)->get();
        return view('payments.payment-batch', compact('voucher_list'));
    }

    public function paymentAdvice($payment_voucher_id)
    {
        $voucher = PaymentVoucher::with(['applicant'])
            ->where('id','=',$payment_voucher_id)
            ->first();
        $applicant_payments = PaymentVoucher::with(['benefit'])
            ->where('applicant_ref_no','=',$voucher->applicant_ref_no)
            ->get();
        return view('payments.payment-advice', compact('voucher','applicant_payments'));
    }
}
