<?php

namespace App\Http\Controllers;

//use App\Classes\ToastNotification;
use App\Models\Permission;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

class PermissionsController extends Controller
{
    //
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(){
        $permissions = Permission::all();
        return view('permissions.index', compact('permissions'));
    }

    public function create(Request $request){

        $this->validate($request,[
            'permission_name' => 'required|unique:permissions,name|string|min:3',
            'display_name' => 'nullable|string|min:3',
            'access_type' => 'required|string|min:3',
            'module' => 'required|string|min:3',
            'permission_desp' => 'nullable|string|min:3',
        ]);
        try{
            $createPermission = new Permission();
            $createPermission->name         = Str::slug(strtolower($request->permission_name));
            $createPermission->display_name = $request->display_name; // optional
            $createPermission->description  = $request->permission_desp; // optional
            $createPermission->access_type  = $request->access_type; // optional
            $createPermission->module  = $request->module; // optional
            $createPermission->save();
            $message = 'The Permission has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'The Role may already exist or check enter appropriate data!'.$ex->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function edit(Request $request)
    {
        $permission = Permission::find($request->id);
        return response()->json($permission);
    }

    public function update(Request $request)
    {
        $this->validate($request,[
            'id' => 'required|numeric|min:1',
            'permission_name' => 'required|unique:permissions,name,'.$request->id.'|string|min:3',
            'display_name' => 'nullable|string|min:3',
            'access_type' => 'required|string|min:3',
            'module' => 'required|string|min:3',
            'permission_desp' => 'nullable|string|min:3'
        ]);

        try{
            $updatePermission = Permission::find($request->id);
            $updatePermission->name         = Str::slug(strtolower($request->permission_name));
            $updatePermission->display_name = $request->display_name; // optional
            $updatePermission->description  = $request->permission_desp; // optional
            $updatePermission->access_type  = $request->access_type; // optional
            $updatePermission->module  = $request->module; // optional
            $updatePermission->save();
            $message = 'The Permission has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'The Permission may already exist or check enter appropriate data!'.$ex->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }
        return back();
    }
}
