<?php

namespace App\Mail;

use App\Models\Deployment;
use App\Models\User;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;

class BeneficiaryInvitation extends Mailable
{
    use Queueable, SerializesModels;

    protected string|null $code;
    protected string $role;
    protected User $user;
    protected Deployment $deployment;
    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct(User $user,Deployment $deployment,$role,$code=null)
    {
        $this->role = $role;
        $this->code = $code;
        $this->user = $user;
        $this->deployment = $deployment;
    }

    /**
     * Get the message envelope.
     *
     * @return \Illuminate\Mail\Mailables\Envelope
     */
    public function envelope()
    {
        return new Envelope(
            subject: 'Invitation to join '.env('APP_NAME'),
        );
    }

    /**
     * Get the message content definition.
     *
     * @return \Illuminate\Mail\Mailables\Content
     */
    public function content()
    {
        return new Content(
            view: 'emails.beneficiary-invitation',
            with:[
                'header_1'  => 'You can enjoy some benefits, '.$this->user->firstname.'!',
                'code'  => $this->code,
                'user'  => $this->user,
                'deployment'  => $this->deployment,
                'role'  => $this->role,
            ]
        );
    }

    /**
     * Get the attachments for the message.
     *
     * @return array
     */
    public function attachments()
    {
        return [];
    }
}
