<?php

namespace App\Mail;

use App\Models\Deployment;
use App\Models\DeploymentInvitation;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;

class SendInstanceInvitation extends Mailable
{
    use Queueable, SerializesModels;

    protected DeploymentInvitation $invitation;
    protected Deployment $deployment;
    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct(DeploymentInvitation $invitation,Deployment $deployment)
    {
        $this->invitation = $invitation;
        $this->deployment = $deployment;
    }

    /**
     * Get the message envelope.
     *
     * @return \Illuminate\Mail\Mailables\Envelope
     */
    public function envelope()
    {
        return new Envelope(
            subject: 'Collaborator Invitation',
        );
    }

    /**
     * Get the message content definition.
     *
     * @return \Illuminate\Mail\Mailables\Content
     */
    public function content()
    {
        return new Content(
            view: 'emails.instance-invitation',
            with: [
                'header_1'   => 'Hello Collaborator!',
                'invitation'   => $this->invitation,
                'deployment'   => $this->deployment
            ]
        );
    }

    /**
     * Get the attachments for the message.
     *
     * @return array
     */
    public function attachments()
    {
        return [];
    }
}
