<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Announcement extends Model
{
    use HasFactory,SoftDeletes;

    protected $fillable = ['title','content','status','publish_date','recipients','featured_image','user_id',
        'pending_alert','created_at','updated_at','visibility'];

    public function addedBy()
    {
        return $this->belongsTo(User::class,'user_id');
    }

    public function scopePublished($query)
    {
        return $query->where('status','=','Publish')->whereDate('publish_date','<=',date('Y-m-d H:i:s',strtotime(now())));
    }

    public function scopeIsPendingAlert($query)
    {
        return $query->where('status','=','Publish')
            ->whereDate('publish_date','<=',now())
            ->where('pending_alert','=',1);
    }
}
