<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use OwenIt\Auditing\Contracts\Auditable;

class Benefit extends Model implements Auditable
{
    use HasFactory, \OwenIt\Auditing\Auditable;

    protected $guarded = [];
    protected $casts = [
        'benefit_types'      =>  'array',
        'supporting_documents'      =>  'array',
        'gender_criteria'      =>  'array',
        'employment_criteria'      =>  'array',
        'residences_id'      =>  'array',
        'residence_criteria'      =>  'array',
        'employment_type_criteria'      =>  'array',
        'nationalities'      =>  'array',
        'concurrent_benefits'      =>  'array',
        'impairments'      =>  'array'
    ];

    public function scopeLiveBenefits($query)
    {
        return $query->where(function ($query) {
            $query->whereRaw("
                CASE 
                    WHEN LENGTH(end_period) = 7 THEN STR_TO_DATE(CONCAT(end_period, '-01'), '%Y-%m-%d') >= CURDATE()
                    WHEN LENGTH(end_period) = 10 THEN STR_TO_DATE(end_period, '%m/%d/%Y') >= CURDATE()
                    ELSE false
                END
            ");
        });
    }

    public function deployment() {
        return $this->belongsTo(Deployment::class,'instance_id');
    }

    public function form() {
        return $this->belongsTo(ApplicationForm::class,'form_id','id');
    }

    public function applications() {
        return $this->hasMany(Application::class,'benefit_id');
    }

    public function approvedApplications() {
        return $this->applications()->where('status','=','Approved')->count();
    }

    public function _class() {
        return $this->belongsTo(Lookup::class,'class','id');
    }

    public function _type() {
        return $this->belongsTo(BenefitType::class,'type','id');
    }

    public function createdBy() {
        return $this->belongsTo(User::class,'created_by','id');
    }

    public function paymentSchedule() {
        return $this->hasOne('App\Models\PaymentSchedule','benefit_id');
    }

    public function payAmounts() {
        return $this->hasMany(BenefitAmount::class,'benefit_id');
    }

    public function payrolls() {
        return $this->hasMany('App\Models\Payroll','benefit_id');
    }

    public function scopeActive($query) {
        return $query->where('archived',0);
    }

    public function scopeActiveMonthAmount($query) {
        return $query->payAmounts->where('month',date('m',strtotime(now())))->where('year',date('Y',strtotime(now())))->first();
    }

    public function inventoryItems()
    {
        return $this->belongsToMany(InventoryItem::class, 'benefit_inventory_item', 'benefit_id', 'inventory_item_id');
    }
}
