<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;

class CaseManagement extends Model
{
    use HasFactory, SoftDeletes;

    protected $table = 'cases';

    protected $casts = [
        'user_ids'   => 'array',
        'role_ids'   => 'array',
    ];

    public function owner() {
        return $this->belongsTo(User::class,'user_id');
    }

    public function _category() {
        return $this->belongsTo(Lookup::class,'category');
    }

    public function _type() {
        return $this->belongsTo(Lookup::class,'type');
    }

    public function comments() {
        return $this->hasMany(CasesCommentsStage::class,'case_id');
    }

    public function lastCommentDate() {
        $comment = DB::table('cases')->join('cases_comments_stages','cases_comments_stages.case_id','=','cases.id')
            ->where('cases.id','=',$this->id)
            ->orderByDesc('cases_comments_stages.updated_at')->pluck('cases_comments_stages.updated_at')->take(1);

        if (isset($comment[0])) return $comment[0];

        return $this->updated_at;
//        return $this->comments()->orderBy('updated_at','desc')->pluck('updated_at')->take(1);
    }

    public function beneficiary() {
        return $this->belongsTo(Applicant::class,'applicant_number','regID');
    }

    public function canAccess() {
        if (auth()->user()->hasRole('beneficiary')) {
            if (isset($this->owner)) {
                if ($this->owner->id == auth()->id()) return true;
            }
            if (isset($this->beneficiary)) {
                if ($this->beneficiary->regID == auth()->user()->applicant_number) return true;
            }
            return false;
        }
        else {
            if (auth()->user()->deployments->where('id','=',$this->instance_id)->count() == 1) return true;
        }
        return false;
    }

    // public function markRead()
    // {
    //     $receivers = GeneralNotification::whereJsonContains('receivers',auth()->id())
    //         ->where('table','=','cases')
    //         ->where('subject_id','=',$this->id)->pluck('receivers')
    //         ->first();

    //     $new_receivers = findReplaceItem(auth()->id(),$receivers);
    //     DB::table('general_notifications')
    //         ->where('table','=','cases')
    //         ->where('subject_id','=',$this->id)
    //         ->update(['receivers'=>$new_receivers]);
    // }

    public function scopeNewCases($query) {
        return $query->where('stage','Opened');
    }

    public function scopeInProgressCases($query) {
        return $query->where('stage','Pending Review');
    }

    public function scopeOnHoldCases($query) {
        return $query->where('stage','On Hold');
    }

    public function scopeClosedCases($query) {
        return $query->where('stage','Close');
    }
}
