<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Support\Str;
use OwenIt\Auditing\Contracts\Auditable;

class Client extends Model implements Auditable
{
    use HasFactory,\OwenIt\Auditing\Auditable;

    protected $guarded = [];

    public function owner()
    {
        return $this->belongsTo(User::class,'admin');
    }

    public function type()
    {
        return $this->belongsTo(Lookup::class,'field');
    }

    public function deployments()
    {
        return $this->hasMany(Deployment::class);
    }

    public function country()
    {
        return $this->belongsTo(Country::class);
    }

    public function collaborators()
    {
        return $this->hasMany(ClientCollaborators::class);
    }

    public function scopeActive($query)
    {
        return $query->where('status','=',1);
    }

    public function isChannelPartner():bool
    {
        if (Str::slug($this->field) == 'channel_partner' || $this->field == 'Channel Partner') return true;
        else return false;
    }

    public function subscriptions()
    {
        return $this->hasMany(ClientSubscription::class,'client_id');
    }

    public function hasActiveSubscription():bool
    {
        if ($this->subscriptions()->where('ends_at', '>', now())->orderByDesc('created_at')->count() > 0) return true;
        return false;
    }

    public function users():BelongsToMany
    {
        return $this->belongsToMany(User::class);
    }

    public function brand()
    {
        return $this->hasOne(ClientBranding::class);
    }
}
