<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class ClientSubscription extends Model
{
    use HasFactory;

    protected $fillable = ['client_id','plan_slug','amount','starts_at','ends_at'];

    public function client()
    {
        return $this->belongsTo(Client::class,'client_id');
    }

    public function plan($key=null)
    {
        $plan = null;

        if (!is_null($key)) {
            if (isset(config('options.channel_partner_plans')[$this->plan_slug][$key])) {
                $plan = config('options.channel_partner_plans')[$this->plan_slug][$key];
            }
        }
        else {
            if (isset(config('options.channel_partner_plans')[$this->plan_slug])) {
                $plan = config('options.channel_partner_plans')[$this->plan_slug];
            }
        }

        return $plan;
    }

    public function scopeFindActive(Builder $builder): Builder
    {
        return $builder->where('ends_at', '>', now());
    }
//    public function active($query)
//    {
//        return $query->where('ends_at', '>', now());
//    }

    public function scopeFindEndedPeriod(Builder $builder): Builder
    {
        return $builder->where('ends_at', '<=', now());
    }

    public function canceled(): bool
    {
        return $this->canceled_at ? Carbon::now()->gte($this->canceled_at) : false;
    }

    /**
     * Check if subscription period has ended.
     *
     * @return bool
     */
    public function ended(): bool
    {
        return $this->ends_at ? Carbon::now()->gte($this->ends_at) : false;
    }

    /**
     * Check if subscription is active.
     *
     * @return bool
     */
    public function active(): bool
    {
        return ! $this->ended();
    }

    public function cancel($immediately = false)
    {
        $this->canceled_at = Carbon::now();

        if ($immediately) {
            $this->ends_at = $this->canceled_at;
        }

        $this->save();

        return $this;
    }
}
