<?php

namespace App\Models;

use App\Models\Traits\NotifiedTrait;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;
use OwenIt\Auditing\Contracts\Auditable;

class Customer extends Authenticatable implements Auditable
{
    use HasApiTokens, HasFactory, Notifiable;
    use \OwenIt\Auditing\Auditable;
    use NotifiedTrait;
    use SoftDeletes;

    protected $fillable = [
        'mobile',
        'email',
        'password',
        'verification_code',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
//        'account_verified_at' => 'datetime',
    ];

    /**
     * Get all application processes.
     */
    public function application_processes()
    {
        return $this->morphMany(ApplicationsProcessing::class, 'applicable');
    }

    /**
     * Get all applications.
     */
    public function applicationsAdded()
    {
        return $this->morphMany(Application::class, 'applicationable');
    }

    public function applications() {
        return $this->hasMany(Application::class,'applicant_number','applicant_number');
    }

    public function applicant() {
        return $this->hasOne(Applicant::class,'regID','applicant_number');
    }

    public function bankDetails() {
        return $this->hasManyThrough(Bank::class,Applicant::class,'regID','applicant_number');
    }

    public function documents() {
        return $this->hasManyThrough(ApplicantDocument::class,Applicant::class,'regID','applicant_number');
    }

}
