<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Http;
use OwenIt\Auditing\Contracts\Auditable;
use Rinvex\Subscriptions\Services\Period;
use Rinvex\Subscriptions\Traits\HasPlanSubscriptions;

class Deployment extends Model implements Auditable
{
    use HasFactory,SoftDeletes;
    use HasFactory,HasPlanSubscriptions,SoftDeletes;
    use \OwenIt\Auditing\Auditable;


    protected $guarded = [];

    protected $casts = [
        'configurations'    =>  'array',
        'applicant_code_format'    =>  'array'
    ];

    public function invitations():HasMany
    {
        return $this->hasMany(DeploymentInvitation::class);
    }

    public function client()
    {
        return $this->belongsTo(Client::class, 'client_id');
    }
    public function createdBy()
    {
        return $this->belongsTo(User::class,'created_by');
    }
    public function admin()
    {
        return $this->belongsTo(User::class,'email','email');
    }

    public function deploymentLogs(){
        return $this->hasMany(DeploymentsLog::class);
    }
    public function applicants()
    {
        return $this->belongsToMany(Applicant::class)->withTimestamps();
    }

    public function benefits(){
        return $this->hasMany(Benefit::class,'instance_id');
    }

    public function users(){
        return $this->belongsToMany(User::class);
    }

    public function cases(){
        return $this->hasMany(CaseManagement::class,'instance_id');
    }

    public function applications(){
        return $this->hasMany(Application::class,'instance_id');
    }

    public function applicantCodeGen(){
        return $this->hasOne(DeploymentApplicantGenCode::class);
    }

    public function activeApplicantCodeGen(){
        return $this->applicantCodeGens()->where('status','=',1)->first();
    }

    public function isReadyForDeployment()
    {
        if (!is_null($this->country_name)) {
            return true;
        }
        return false;
    }

    public function scopeGetConfig($query,$key) {
        return isset($query->$key) ? $data = $query->$key: $data = '';
    }

    /**
     * Subscribe subscriber to a new plan.
     */
    public function newPlanSubscription($subscription, Plan $plan, Carbon $startDate = null)
    {
        $subscription_interval = $plan->invoice_period;
        if ($this->subscription_interval != null) $subscription_interval = $this->subscription_interval;
        $trial = new Period($plan->trial_interval, $plan->trial_period, $startDate ?? now());
        $period = new Period($plan->invoice_interval, $subscription_interval, $trial->getEndDate());

        return $this->planSubscriptions()->create([
            'name' => $subscription,
            'plan_id' => $plan->getKey(),
            'trial_ends_at' => $trial->getEndDate(),
            'starts_at' => $period->getStartDate(),
            'ends_at' => $period->getEndDate(),
        ]);
    }


    /**
     * Subscribe subscriber to a new plan.
     */
    public function setNewPeriod($invoice_interval = 'month',$invoice_period = 1, Carbon $startDate = null)
    {
        $period = new Period($invoice_interval, $invoice_period, $startDate);

        $this->planSubscription($this->code)->starts_at = $period->getStartDate();
        $this->planSubscription($this->code)->ends_at = $period->getEndDate();

        $this->planSubscription($this->code)->save();
        return $period->getEndDate();
    }

    public function items(){
        return $this->hasMany(InventoryItem::class);
    }
}
