<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class LookupCode extends Model
{
    use HasFactory;

    protected $table = 'lookup_codes';

    public function options()
    {
        return $this->hasMany('App\Models\Lookup','lookup_code_id');
    }

    public function scopeActiveOptions($query,$lookup_id)
    {
        return $query->find($lookup_id)->options->where('activeFlag',true)->pluck('fullName','id');
    }

    public function scopeActiveOptionsFull($query,$lookup_id)
    {
        return $query->find($lookup_id)->options->where('activeFlag',true);
    }

    public function scopeActiveApplicationProcessOptions($query)
    {
        return $query->find(7)->options->where('activeFlag',true)->pluck('fullName','id');
    }

    public function scopeActivePayrollProcessOptions($query)
    {
        return $query->find(config('settings.payment_process_lookupCode'))->options->where('activeFlag',true)->pluck('fullName','id');
    }

    public function scopeActiveComplaintProcessOptions($query)
    {
        return $query->find(config('settings.complaint_process_lookupCode'))->options->where('activeFlag',true)->pluck('fullName','id');
    }

    public function scopeActiveApprovalProcessOptions($query)
    {
        return $query->where('is_process',true)->options->where('activeFlag',true)->pluck('fullName','id');
    }

    public function scopeActiveApprovalProcessesOptions($query)
    {
        return $query->join('lookups','lookups.lookup_code_id','=','lookup_codes.id')
            ->where('lookup_codes.is_process',true)
            ->where('lookups.activeFlag',true)
            ->where('lookup_codes.activeFlag',true)
            ->select('fullName','lookups.id','lookupName')
            ->orderBy('lookupName','ASC')
            ->get();
    }

    public function scopeAllOptions($query,$lookup_id)
    {
        return $query->find($lookup_id)->options->pluck('fullName','id');
    }
}
