<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use OwenIt\Auditing\Contracts\Auditable;

class Plan extends \Rinvex\Subscriptions\Models\Plan implements Auditable
{
    use HasFactory;
    use \OwenIt\Auditing\Auditable;

    protected $fillable = [
        'slug',
        'name',
        'description',
        'is_active',
        'price',
        'year_discount_percent',
        'popular',
        'signup_fee',
        'currency',
        'trial_period',
        'trial_interval',
        'invoice_period',
        'invoice_interval',
        'grace_period',
        'grace_interval',
        'prorate_day',
        'prorate_period',
        'prorate_extend_due',
        'active_subscribers_limit',
        'sort_order',
        'country_id',
        'is_custom',
    ];

    public function __construct(array $attributes = [])
    {
        $this->setTable('plans');
        $this->mergeRules([
            'slug' => 'required|alpha_dash|max:150|unique:plans,slug',
            'name' => 'required|string|strip_tags|max:150',
            'description' => 'nullable|string|max:32768',
            'is_active' => 'sometimes|boolean',
            'price' => 'required|numeric',
            'year_discount_percent' => 'required|numeric',
            'signup_fee' => 'required|numeric',
            'currency' => 'required|alpha|size:3',
            'trial_period' => 'sometimes|integer|max:100000',
            'trial_interval' => 'sometimes|in:hour,day,week,month,year',
            'invoice_period' => 'sometimes|integer|max:100000',
            'invoice_interval' => 'sometimes|in:hour,day,week,month,year',
            'grace_period' => 'sometimes|integer|max:100000',
            'grace_interval' => 'sometimes|in:hour,day,week,month,year',
            'sort_order' => 'nullable|integer|max:100000',
            'prorate_day' => 'nullable|integer|max:150',
            'prorate_period' => 'nullable|integer|max:150',
            'prorate_extend_due' => 'nullable|integer|max:150',
            'active_subscribers_limit' => 'nullable|integer|max:100000',
        ]);

        parent::__construct($attributes);
    }

    public function hasFeatureByName($feature_name)
    {
        if($this->features->where('name',$feature_name)->first()) {
            return true;
        }
        else {
            return false;
        }
    }

    public function country()
    {
        return $this->belongsTo(Country::class);
    }

    public function countryCosts()
    {
        return $this->hasMany(CountryPlan::class);
    }

    public function countryPricing()
    {
        return $this->hasMany(CountryPlanPricing::class);
    }
}
