<?php

namespace App\Models;

use Laratrust\Models\LaratrustRole;
use OwenIt\Auditing\Contracts\Auditable;


class Role extends LaratrustRole implements Auditable
{
	use \OwenIt\Auditing\Auditable;

	protected $guarded = [];


    protected $visible = [ 'id', 'display_name','lead' ];

    public function toArray()
    {
        $attributes = $this->attributesToArray();
        $attributes = array_merge($attributes, $this->relationsToArray());
        unset($attributes['pivot']['created_at']);
        return $attributes;
    }

    public function users()
    {
        return $this->belongsToMany(User::class)
            ->wherePivotIn('instance_id', [getActiveInstance()]);
    }

    public function supervisor()
    {
        return $this->belongsTo(Role::class,'lead','id');
    }


    public function parent()
    {
        return $this->belongsTo(Role::class, 'lead');
    }

    public function children()
    {
        return $this->hasMany(Role::class, 'lead','id');
    }

    public function getHigherRoles()
    {
        $parents = [];
        $currentParent = $this->parent;

        while ($currentParent) {
            $parents[] = $currentParent;
            $currentParent = $currentParent->parent;
        }

        return $parents;
    }
}
