<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class TeamManagementMember extends Model
{
    use HasFactory, SoftDeletes;

    protected $guarded = [];
    protected $casts = [
        'languages' => 'array'
    ];

    public function createdBy()
    {
        return $this->belongsTo(User::class,'created_by');
    }

    public function updatedBy()
    {
        return $this->belongsTo(User::class,'updated_by');
    }

    public function _gender()
    {
        return $this->belongsTo(Lookup::class,'gender');
    }

    public function _education_level()
    {
        return $this->belongsTo(Lookup::class,'education_level');
    }

    public function teams()
    {
        return $this->hasManyThrough(Team::class,TeamMember::class);
    }
    
    public function teamMemberships()
    {
        return $this->hasMany(TeamMember::class,'user_id');
    }

    public function scopeActive($query)
    {
        return $query->where('status','=',1);
    }
}
