<x-app-layout>
    <x-slot name="breadcrumb">
        <ol class="breadcrumb bg-transparent mb-0">
            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('dashboard') }}">Dashboard</a></li>
            <li class="breadcrumb-item active" aria-current="page">{{$page_title}}</li>
        </ol>
    </x-slot>
    <!-- start: page toolbar -->
    <x-slot name="page_heading">
        <div class="row align-items-center">
            <div class="col-auto page-header-title">
                <h1 class="color-900 mt-1 mb-0">{{$page_title}}</h1>
                <small class="text-muted">
                    This module serves as the central hub for managing scheme members' account profiles.
                    <br>It facilitates essential tasks such as adding new members, suspending accounts,
                    updating member details, and handling password resets.
                </small>
            </div>
            <div class="col d-flex justify-content-lg-end mt-2 mt-md-0">
                <div class="p-2 me-md-3">
                    <div>
                        <span class="h6 mb-0">{{easyCount($active)}}</span>
                            @if($active > 0 && $applicants > 0)
                                <small class="text-secondary"><i class="fa fa-angle-up"></i>
                                    {{round($active*100/$applicants)}}%</small>
                            @endif
                        
                    </div>
                    <small class="text-muted text-uppercase">Active</small>
                </div>
                <div class="p-2 me-md-3">
                    <div>
                        <span class="h6 mb-0">{{easyCount($inactive)}}</span>
                        @if($inactive > 0 && $applicants > 0)
                            <small class="text-secondary"><i class="fa fa-angle-up"></i>
                                {{round($inactive*100/$applicants)}}%</small>
                        @endif
                    </div>
                    <small class="text-muted text-uppercase">Inactive</small>
                </div>
            </div>
        </div> <!-- .row end -->
    </x-slot>

    <div class="page-body px-xl-4 px-sm-2 px-0 py-lg- py-1 mt-0 mt-lg-">
        <div class="container-fluid">
            @if(session()->has('message'))
                <div class="mb-5">
                    @if(session()->get('status') == 'success')
                        <div role="alert" class="alert alert-success">{{ session()->get('message') }}</div>
                    @else
                        <div role="alert" class="alert alert-danger">{{ session()->get('message') }}</div>
                    @endif
                </div>
            @endif

            <div class="row g-3">
                @if(canDo('administrator','search-beneficiary|member-records-update|add-beneficiary'))
                <div class="col-12">
                    <div class="card">
                        <div class="card-body">
                            
                            <form action="{{route('applicant.getApplicantInfo')}}" method="get">
                                @csrf
                                <div class="input-group">
                                    @if(canDo('administrator','search-beneficiary'))
                                        <select name="query_option" id="query_option" class="form-select" required>
                                            @php
                                                $query_option = request()->get('query_option');
                                                if (!is_null($query)) {
                                                    $query_option = $query;
                                                }
                                            @endphp
                                            <option>Query By</option>
                                            <option
                                                value="beneficiary" {{ ($query_option == 'beneficiary')?'selected':'' }}>{{ __('Member Status') }}</option>
                                            <option
                                                value="refID" {{ ($query_option == 'refID')?'selected':'' }}>{{ __('Ref. ID') }}</option>
                                            <option
                                                value="dobs" {{ ($query_option == 'dobs')?'selected':'' }}>{{ __('Date of Birth') }}</option>
                                        </select>

                                        <select name="beneficiary_option" id="beneficiary_option"
                                                class="form-select" required style="display: {{ ($query_option == 'beneficiary')?'block':'none' }}">
                                            @php
                                                $beneficiary_option = request()->get('beneficiary_option');
                                                if (!is_null($query)) {
                                                    $query_option = $query;
                                                }
                                            @endphp
                                            <option
                                                value="active" {{ ($beneficiary_option == 'active')?'selected':'' }}>{{ __('Active') }}</option>
                                            <option
                                                value="archived" {{ ($beneficiary_option == 'archived')?'selected':'' }}>{{ __('Archived') }}</option>
                                            <option
                                                value="ended" {{ ($beneficiary_option == 'ended')?'selected':'' }}>{{ __('Ended') }}</option>
                                        </select>

                                        <input type="text" placeholder="{{settings()->get('applicant_title')}} Ref. ID" class="form-control"
                                               value="{{ request()->get('refID')}}" name="refID" id="refID" style="display: {{ ($query_option == 'refID')?'block':'none' }}">

                                        <input type="text" class="form-control datepicker" value="{{ request()->get('dobs') }}"
                                               required name="dobs" id="dobs" readonly="readonly"
                                               placeholder="Select date" style="display: {{ ($query_option == 'dobs')?'block':'none' }}">

                                        <select name="gender" id="gender" class="form-select">
                                            <option value="">{{ __('-- Gender --') }}</option>
                                            <option value="Male" {{ (request()->get('gender') == 'Male')?'selected':'' }}>{{ __('Male') }}</option>
                                            <option value="Female" {{ (request()->get('gender') == 'Female')?'selected':'' }}>{{ __('Female') }}</option>
                                        </select>

                                        <input type='text' autocomplete="off" class="form-control" id="date_range" name="date_range" placeholder="Request Period"
                                                                        value="{{request()->get('date_range')}}"/>

                                        <select name="location" id="location" class="form-select">
                                            <option value="">{{ __('-- Birth Place --') }}</option>
                                            @foreach(getLocations() as $key=>$city)
                                                <option value="{{$key}}" {{ (request()->get('location') == $key)?'selected':'' }}>{{ $city }}</option>
                                            @endforeach
                                        </select>

                                        <button class="btn btn-secondary" type="submit" data-bs-toggle="tooltip"
                                                data-bs-placement="top" title="Search"><i class="fa fa-search"></i></button>
                                        <a href="{{ route('applicant.getApplicantInfo') }}" class="btn btn-light-primary" data-bs-toggle="tooltip" title="Reset"> <i class="bi-arrow-repeat"></i></a>
                                    @endif

                                    @if(false)
                                    @if(canDo('administrator','member-records-update|view-member-record-comment|add-member-record-comment|process-member-record'))
                                        <a href="{{ route('memberRecords.list') }}" class="btn btn-light-success" data-bs-toggle="tooltip" title="Member Records"> <i class="bi-person-lines-fill"></i></a>
                                    @endif
                                    @endif

                                    @if(canDo('administrator','add-beneficiary'))
                                        <a href="{{ route('applicant.applicantRegistration') }}" class="btn btn-primary" data-bs-toggle="tooltip"
                                           data-bs-placement="top" title="Add new Member"><i class="fa fa-plus me-2"></i> {{__('New')}}</a>
                                    @endif
                                </div>
                            </form>

                            <!--
                            <form action="{{route('applicant.getApplicantInfo')}}" method="get">
                                @csrf
                                <div class="input-group">
                                    @if(canDo('administrator','search-beneficiary'))
                                        <select name="query_option" id="query_option" class="form-select" required>
                                            @php
                                                $query_option = request()->get('query_option');
                                                if (!is_null($query)) {
                                                    $query_option = $query;
                                                }
                                            @endphp
                                            <option>Query By</option>
                                            <option value="beneficiary" {{ ($query_option == 'beneficiary')?'selected':'' }}>{{ __('Member Status') }}</option>
                                            <option value="refID" {{ ($query_option == 'refID')?'selected':'' }}>{{ __('Ref. ID') }}</option>
                                            <option value="name" {{ ($query_option == 'name')?'selected':'' }}>{{ __('Name') }}</option>
                                            <option value="dobs" {{ ($query_option == 'dobs')?'selected':'' }}>{{ __('Date of Birth') }}</option>
                                            <option value="county" {{ ($query_option == 'county')?'selected':'' }}>{{ __('County') }}</option>
                                            <option value="pmt" {{ ($query_option == 'pmt')?'selected':'' }}>{{ __('PMT') }}</option>
                                            
                                        </select>

                                        <select name="beneficiary_option" id="beneficiary_option"
                                                class="form-select" required style="display: {{ ($query_option == 'beneficiary')?'block':'none' }}">
                                            @php
                                                $beneficiary_option = request()->get('beneficiary_option');
                                                if (!is_null($query)) {
                                                    $query_option = $query;
                                                }
                                            @endphp
                                            <option
                                                value="active" {{ ($beneficiary_option == 'active')?'selected':'' }}>{{ __('Active') }}</option>
                                            <option
                                                value="archived" {{ ($beneficiary_option == 'archived')?'selected':'' }}>{{ __('Archived') }}</option>
                                            <option
                                                value="ended" {{ ($beneficiary_option == 'ended')?'selected':'' }}>{{ __('Ended') }}</option>
                                        </select>

                                        <select name="county_option" id="county_option" class="form-select" required style="display: {{ ($query_option == 'county')?'block':'none' }}">
                                            <option value="">-- Select County --</option>
                                        </select>

                                        <input type="number" class="form-control" id="pmt_from" required name="pmt_from" placeholder="From" style="display: {{ ($query_option == 'pmt')?'block':'none' }}" value="{{request()->get('pmt_from')}}">
                                        <input type="number" class="form-control" id="pmt_to" required name="pmt_to" placeholder="To" style="display: {{ ($query_option == 'pmt')?'block':'none' }}" value="{{ request() -> get('pmt_to')}}">

                                        <input type="text" name="applicant_name" id="applicant_name" class="form-control" style="display: {{ ($query_option == 'name')?'block':'none' }}" placeholder="Search by Name">

                                        <input type="text" placeholder="{{settings()->get('applicant_title')}} Ref. ID" class="form-control"
                                               value="{{ request()->get('refID')}}" name="refID" id="refID" style="display: {{ ($query_option == 'refID')?'block':'none' }}">

                                        <input type="text" class="form-control datepicker" value="{{ request()->get('dobs') }}"
                                               required name="dobs" id="dobs" readonly="readonly"
                                               placeholder="Select date" style="display: {{ ($query_option == 'dobs')?'block':'none' }}">

                                        <select name="gender" id="gender" class="form-select">
                                            <option value="">{{ __('-- Gender --') }}</option>
                                            <option value="Male" {{ (request()->get('gender') == 'Male')?'selected':'' }}>{{ __('Male') }}</option>
                                            <option value="Female" {{ (request()->get('gender') == 'Female')?'selected':'' }}>{{ __('Female') }}</option>
                                        </select>

                                        <input type='text' autocomplete="off" class="form-control" id="date_range" name="date_range" placeholder="Request Period"
                                                                        value="{{request()->get('date_range')}}"/>

                                        <select name="location" id="location" class="form-select">
                                            <option value="">{{ __('-- Birth Place --') }}</option>
                                            @foreach(getLocations() as $key=>$city)
                                                <option value="{{$key}}" {{ (request()->get('location') == $key)?'selected':'' }}>{{ $city }}</option>
                                            @endforeach
                                        </select>

                                        <button class="btn btn-secondary" type="submit" data-bs-toggle="tooltip"
                                                data-bs-placement="top" title="Search"><i class="fa fa-search"></i></button>
                                        <a href="{{ route('applicant.getApplicantInfo') }}" class="btn btn-light-primary" data-bs-toggle="tooltip" title="Reset"> <i class="bi-arrow-repeat"></i></a>
                                    @endif

                                    @if(false)
                                    @if(canDo('administrator','member-records-update|view-member-record-comment|add-member-record-comment|process-member-record'))
                                        <a href="{{ route('memberRecords.list') }}" class="btn btn-light-success" data-bs-toggle="tooltip" title="Member Records"> <i class="bi-person-lines-fill"></i></a>
                                    @endif
                                    @endif

                                    @if(canDo('administrator','add-beneficiary'))
                                        <a href="{{ route('applicant.applicantRegistration') }}" class="btn btn-primary" data-bs-toggle="tooltip"
                                           data-bs-placement="top" title="Add new Member"><i class="fa fa-plus me-2"></i> {{__('New')}}</a>
                                    @endif
                                </div>

                            </form>
-->
                        </div>
                    </div>
                </div>
                @endif

                <div class="col-12">
                    <div class="table-responsive mt-10">
                            <table id="myDataTable_no_filter" class="table table-sm myDataTable align-middle table-bordered mb-3 custom-table">
                            <thead>
                            <tr>
                                <th style="width: 20px;">#</th>
                                <th>{{__('Member')}}</th>
                                <th>{{__('Bio')}}</th>
                                <th>{{__('Citizenship/Nationality')}}</th>
                                <th>{{__('Other')}}</th>
                                <th class="text-right">{{__('Actions')}}</th>
                            </tr>
                            </thead>
                            <tbody></tbody>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>

    <!--begin::Row-->
    <div class="modal fade" id="staticBackdrop" data-backdrop="static" tabindex="-1" role="dialog"
         aria-labelledby="staticBackdrop" aria-hidden="true">
        <div class="modal-dialog modal-lg" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <h5 class="modal-title" id="exampleModalLabel">{{ settings()->get('applicant_title').__(' Information') }}</h5>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <i aria-hidden="true" class="ki ki-close"></i>
                    </button>
                </div>
                <div class="modal-body" id="info">

                </div>
                <div class="modal-footer">
                    <button type="button" class="btn btn-light-primary font-weight-bold" data-dismiss="modal">Close
                    </button>
                </div>
            </div>
        </div>
    </div>
    <!--end::Row-->
    @include('touring-pages.membership')
    @push('vendor-scripts')
    @endpush
    @push('scripts')
        <script src="{{ asset('assets/js/bundle/dataTables.bundle.js') }}"></script>
{{--        <script src="{{ asset('assets/js/bundle/bootstrapdatepicker.bundle.js') }}"></script>--}}
{{--        <script src="{{ asset('assets/js/bundle/datetimepicker.bundle.js') }}"></script>--}}
        <script src="{{ asset('assets/js/bundle/daterangepicker.bundle.js') }}"></script>
        <script src="{{ asset('assets/js/custom.js') }}"></script>
        @if (canTour('membership'))
            <script>
                $(window).on('load', function() {
                    $('#take-a-tour').modal('show');
                });

                function skipTour(page) {
                        var path = "{{route('on_boarding.skipTour')}}";
                        $.ajax({
                            url: path,
                            type: 'GET',
                            data:{'page':page},
                            beforeSend: function () {
                            },
                            success: function (data) {
                                console.log('Skipped');
                            }
                        });
                    }
            </script>
        @endif
        <script>
            $('.datepicker').datepicker({});
            $('#query_option').change(function () {
                if (this.value == 'refID') {
                    $('#dobs').hide();
                    $('#dobs').prop({disabled: true, required: false});
                    $('#refID').show();
                    $('#refID').prop({disabled: false, required: true});
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                    $('#applicant_name').hide();
                    $('#applicant_name').prop({disabled: true, required: false});
                } else if (this.value == 'dobs') {
                    $('#dobs').show();
                    $('#dobs').prop({disabled: false, required: true});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                    $('#applicant_name').hide();
                    $('#applicant_name').prop({disabled: true, required: false});
                } else if (this.value == 'beneficiary') {
                    $('#beneficiary_option').show();
                    $('#beneficiary_option').prop({disabled: false, required: true});
                    $('#dobs').hide();
                    $('#dobs').prop({disabled: true, required: false});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                    $('#applicant_name').hide();
                    $('#applicant_name').prop({disabled: true, required: false});
                }
                else if (this.value == 'pmt') {
                    $('#pmt_from').show();
                    $('#pmt_from').prop({disabled: false, required: true});
                    $('#pmt_to').show();
                    $('#pmt_to').prop({disabled: false, required: true});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                    $('#applicant_name').hide();
                    $('#applicant_name').prop({disabled: true, required: false});
                }
                else if (this.value == 'name') {
                    $('#applicant_name').show();
                    $('#applicant_name').prop({disabled: false, required: true});
                    $('#pmt_from').hide();
                    $('#pmt_from').prop({disabled: true, required: false});
                    $('#pmt_to').hide();
                    $('#pmt_to').prop({disabled: true, required: false});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                }
                
                else if (this.value == 'county') {
                    $('#county_option').show();
                    $('#county_option').prop({disabled: false, required: true});
                    $('#pmt_from').hide();
                    $('#pmt_from').prop({disabled: true, required: false});
                    $('#pmt_to').hide();
                    $('#pmt_to').prop({disabled: true, required: false});
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#dobs').hide();
                    $('#dobs').prop({disabled: true, required: false});
                    $('#location').hide();
                    $('#location').prop({disabled: true, required: false});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#applicant_name').hide();
                    $('#applicant_name').prop({disabled: true, required: false});
                }
                
                else {
                    $('#beneficiary_option').hide();
                    $('#beneficiary_option').prop({disabled: true, required: false});
                    $('#dobs').hide();
                    $('#dobs').prop({disabled: true, required: false});
                    $('#refID').hide();
                    $('#refID').prop({disabled: true, required: false});
                    $('#county_option').hide();
                    $('#county_option').prop({disabled: true, required: false});
                }
            });

            $(function() {
                var start = moment().startOf('year');
                var end = moment();
                // function cb(start, end) {
                //     $('input[name="date_range"]').html();
                // }
                $('input[name="date_range"]').daterangepicker({
                    locale: {
                        cancelLabel: 'Clear'
                    },
                    startDate: start,
                    endDate: end,
                    ranges: {
                        'Today': [moment(), moment()],
                        'Yesterday': [moment().subtract(1, 'days'), moment().subtract(1, 'days')],
                        'Last 7 Days': [moment().subtract(6, 'days'), moment()],
                        'Last 30 Days': [moment().subtract(29, 'days'), moment()],
                        'This Month': [moment().startOf('month'), moment().endOf('month')],
                        'Last Month': [moment().subtract(1, 'month').startOf('month'), moment().subtract(1, 'month').endOf('month')],
                        'This Year': [moment().startOf('year'), moment()],
                        'Last Year': [moment().subtract(12, 'month').startOf('year'), moment().subtract(12, 'month').endOf('year')]
                    }
                });
                // cb(start, end);

                $('input[name="date_range"]').on('cancel.daterangepicker', function(ev, picker) {
                    $('input[name="date_range"]').val('');
                });
            });

            // $('#myDataTable_no_filter').addClass('nowrap').dataTable({
            //     responsive: false,
            //     searching: false,
            //     paging: false,
            //     ordering: false,
            //     info: false,
            // });

            @if(!is_null($query))
                @php if ($query == 'dobs') $value = date('Y-m-d',strtotime($value)); @endphp
                const APPLICATIONS_AJAX = "{{ route('applicant.getApplicationsInfoList',[
                    'query_option'=>request()->get('query_option'),
                    'refID'=>request()->get('refID'),
                    'beneficiary_option'=>request()->get('beneficiary_option'),
                    'gender'=>request()->get('gender'),
                    'date_range'=>request()->get('date_range'),
                    'location'=>request()->get('location'),
                    'dobs'=>request()->get('dobs')
                    ]) }}";
            @else
                const APPLICATIONS_AJAX = "{{ route('applicant.getApplicationsInfoList') }}";
            @endif
        </script>
        <script>

            function getApplicantInfo(id) {
                var path = "{{route('applicant.viewApplicantInfo')}}";
                $.ajaxSetup({
                    headers: {
                        'X-CSRF-Token': $('meta[name=_token]').attr('content')
                    }
                });
                $.ajax({
                    url: path,
                    type: 'GET',
                    data: {app_id: id},
                    beforeSend: function () {
                    },
                    success: function (data) {
                        $('#info').empty();
                        $('#info').html(data.theView);
                    },
                    complete: function () {
                        $.getScript("{{ asset('assets/js/customjs/custom.js') }}", function () {
                            console.log('Reset');
                        });
                    }
                    ,
                    error: function (data) {
                    }
                });
            }
        </script>

        <script src="{{ asset('assets/js/customjs/custom-applicant-registration-datatable.js') }}"></script>
    @endpush
</x-app-layout>
