<x-app-layout>
    @push('vendor-css')
        <link rel="stylesheet" href="{{ asset('assets/plugins/custom/datatables/datatables.bundle.css') }}">
    @endpush
    <x-slot name="page_title">
        {{ __('Process Stages Setup') }}
    </x-slot>
    <x-slot name="breadcrumb">
        <ul class="breadcrumb breadcrumb-transparent breadcrumb-dot font-weight-bold p-0 my-2 font-size-sm">
            <li class="breadcrumb-item text-muted">
                <a href="{{ route('dashboard') }}" class="text-muted">{{ __('Dashboard') }}</a>
            </li>
            <li class="breadcrumb-item text-muted">
                <a href="" class="text-muted">{{ __('Process Stages Setup') }}</a>
            </li>
        </ul>
    </x-slot>

    <div class="row">
        <div class="col-md-12">
            <div class="card card-custom">
                <div class="card-header">
                    <h3 class="card-title">{{ __('Assign Email Notification Template') }}</h3>
                    <div class="card-toolbar">
                        <a href="{{ route('settings.stage_emails') }}" class="btn btn-sm btn-success font-weight-bold">
                            <i class="la la-refresh"></i> {{ __('Refresh') }}</a>
                    </div>
                </div>

                <div class="card-body" id="theForm">
                    <form method="POST" action="{{ route('settings.stage_emails.save') }}">
                        <div class="row">
                            @csrf
                            <div class="col-md-2">
                                <div class="form-group{{ $errors->has('order') ? ' form-control-warning' : '' }}">
                                    <label for="order">{{ __('Order No.') }}<span class="required">*</span></label>
                                  <input type="number" min="1" required class="form-control"  value="1" name="order" id="order">
                                </div>
                            </div>
                            <div class="col-md-10">
                                <div class="form-group{{ $errors->has('status') ? ' form-control-warning' : '' }}">
                                    <label for="status">{{ __('Approval Process Stage') }}<span class="required">*</span></label>
                                    <select name="status" id="status" class="form-control select2" style="width: 100%;">
                                        <option></option>
                                        @foreach($stages as $stage)
                                            <option value="{{$stage->id}}">{{$stage->fullName}} <strong class="text-muted"> ~~ {{$stage->lookupName}}</strong></option>
                                        @endforeach
                                    </select>
                                </div>
                            </div>
                            <div class="col-md-8">
                                <div class="form-group{{ $errors->has('email') ? ' form-control-warning' : '' }}">
                                    <label for="email">{{ __('Email Template') }}<span class="required">*</span></label>
                                    <select name="email" id="email" class="form-control select2" style="width: 100%;">
                                        <option></option>
                                        @foreach($email_templates as $email_template)
                                            <option value="{{$email_template->id}}">{{$email_template->title}}</option>
                                        @endforeach
                                    </select>
                                </div>
                            </div>
                            <div class="col-md-4">
                                <div class="form-group{{ $errors->has('roles') ? ' form-control-warning' : '' }}">
                                    <label for="roles">{{ __('Permitted Roles') }}<span class="required">*</span></label>
                                    <select name="roles[]" multiple id="roles" class="form-control select2" style="width: 100%;">
                                        <option></option>
                                        @foreach($roles as $role)
                                            <option value="{{$role->id}}">{{$role->display_name}}</option>
                                        @endforeach
                                    </select>
                                </div>
                            </div>
                        </div>
                        <button type="submit" class="btn btn-primary">
                            <i class="la la-save"></i> {{ __('Save') }}
                        </button>
                    </form>
                </div>
            </div>

            <div class="card card-custom mt-4">
                <div class="card-header">
                    <h3 class="card-title">{{ __('List of Request Status with Notification Emails') }}</h3>
                </div>
                <div class="card-body">
                    <div class="table-responsive">
                        <table class="table table-bordered template-list">
                            <thead>
                            <tr>
                                <th scope="col" style="width: 50px;">{{ __('Order') }}</th>
                                <th scope="col">{{ __('Stage') }}</th>
                                <th scope="col">{{ __('Process Type') }}</th>
                                <th scope="col">{{ __('Email Template') }}</th>
                                <th scope="col">{{ __('Permitted Roles') }}</th>
                                <th scope="col" style="width: 130px;">{{ __('Action') }}</th>
                            </tr>
                            </thead>
                            <tbody>
                            @foreach($stage_emails as $stage_email)
                                <tr>
                                    <td>{{ucwords($stage_email->order)}}</td>
                                    <td>
                                        {{ucwords($stage_email->lookup->fullName)}}
                                    </td>
                                    <td>{{ucwords($stage_email->lookup->lookupCode->lookupName)}}</td>
                                    <td>{{ucwords($stage_email->email_template->title)}}</td>
                                    <td>
                                        @php
                                            $permitted_roles = explode(',',$stage_email->roles);
                                        @endphp
                                        @foreach($roles as $role)
                                            @if(in_array($role->id,$permitted_roles))
                                                <span class="mr-2">- {{$role->display_name}}</span>
                                            @endif
                                        @endforeach
                                    </td>
                                    <td>
                                        {{--<div class="btn-group" role="group" aria-label="Basic example">--}}
                                            <a href="{{\Illuminate\Support\Facades\URL::current()}}#theForm" title="{{__('Edit')}}"
                                               class="btn btn-light-primary btn-xs btn-icon" onclick="edit({{$stage_email->id}})">
                                                <i class="la la-edit"></i>
                                            </a>
                                            <a class="btn btn-danger btn-xs btn-icon" href="{{ route('settings.stage_emails.destroy',[$stage_email->id]) }}"
                                               onclick="event.preventDefault();
                                                       document.getElementById('delete-form-{{$stage_email->id}}').submit();" title="{{__('Delete')}}">
                                                <i class="la la-trash"></i>
                                            </a>

                                            <form id="delete-form-{{$stage_email->id}}" action="{{ route('settings.stage_emails.destroy',[$stage_email->id]) }}"
                                                  method="POST" style="display: none;">
                                                @csrf {{method_field('DELETE')}}
                                            </form>
                                        {{--</div>--}}
                                    </td>
                                </tr>
                            @endforeach
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    @push('vendor-scripts')
        <script src="{{ asset('assets/plugins/custom/datatables/datatables.bundle.js') }}"></script>
    @endpush
    @push('scripts')
        <script>
            $('document').ready(function () {
    
                $('.select2').select2({
                    placeholder: "Select Option",
                    allowClear: true
                });

                $('.template-list').dataTable({
                    order: [[2, 'asc'],[0, 'asc']],
                });
            });
            function edit(key) {
                var path = "{{route('settings.stage_emails.edit')}}";
                $.ajaxSetup(    {
                    headers: {
                        'X-CSRF-Token': $('meta[name=_token]').attr('content')
                    }
                });
                $.ajax({
                    url: path,
                    type: 'GET',
                    data: {id:key},
                    beforeSend: function(){
                        $('#theForm').block({
                            message: '<div class="ft-loader icon-spin font-large-1"></div>',
                            // timeout: 2000, //unblock after 2 seconds
                            overlayCSS: {
                                backgroundColor: '#ccc',
                                opacity: 0.8,
                                cursor: 'wait'
                            },
                            css: {
                                border: 0,
                                padding: 0,
                                backgroundColor: 'transparent'
                            }
                        });
                    },
                    success: function(data){
                        $('#theForm').empty().html(data.theView);
                    },
                    complete:function(){
                        $('.select2').select2({
                            placeholder: "Select Option",
                            allowClear: true
                        });
                    },
                    error: function (data) {
                        console.log(data.field)
                    }
                });
            }
        </script>
    @endpush
</x-app-layout>
