<?php

use App\Http\Controllers\ApiController;
use App\Http\Controllers\AdminApiController;
use Illuminate\Http\Request;
use \App\Http\Controllers\Api\DeploymentApiController;
use \App\Http\Controllers\Api\LocationApiController;
use \App\Http\Controllers\Api\MerchantsApiController;
use \App\Http\Controllers\Api\BenefitTypesApiController;
use \App\Http\Controllers\Api\LookupApiController;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::post('/login', [ApiController::class,'login']);
Route::post('/register', [ApiController::class,'register']);
Route::get('/check-auth', [ApiController::class,'userLoggedIn']);
Route::post('reset-password', [ApiController::class, 'resetPassword']);

Route::get('/get-users-count/{console_code}', [ApiController::class, 'getUsers']);
Route::get('/get-applicants-count/{console_code}', [ApiController::class, 'getApplicants']);
Route::get('/get-applications-count/{console_code}', [ApiController::class, 'getApplications']);

Route::get('/update-project/{console_code}', [ApiController::class, 'updateProject']);

Route::group(['middleware' => ['auth:sanctum']],function () {
    Route::get('profile', [ApiController::class, 'profile']);
    Route::get('verify-status', [ApiController::class, 'verifyStatus']);
    Route::post('connect-application', [ApiController::class, 'connectToApplication']);
    Route::post('verify-code', [ApiController::class, 'verifyCode']);
    Route::get('resend-verify-code', [ApiController::class, 'ReSendVerifyCode']);
    Route::post('new-applicant', [ApiController::class, 'saveCustomerRegistration']); //NEW
    Route::post('update-applicant', [ApiController::class, 'updateCustomerRecord']); //NEW
    Route::post('verify-application', [ApiController::class, 'verifyApplication']);
    Route::get('my-applications', [ApiController::class, 'applications']);
    Route::get('applicant-details', [ApiController::class, 'applicantDetails']);
    Route::get('benefit/{benefit_id}', [ApiController::class, 'getBenefit']);
    Route::get('applicant-completion-level', [ApiController::class, 'getApplicantCompletionLevel']);
    Route::get('get-applicant-eligible-benefits', [ApiController::class, 'getApplicantEligibleBenefits']);
    Route::get('get-applicant-enrolled-benefits', [ApiController::class, 'getApplicantEnrolledBenefits']);
    Route::post('update-profile', [ApiController::class, 'accountInfoUpdate']);
    Route::post('change-password', [ApiController::class, 'changePassword']);
    Route::post('logout', [ApiController::class, 'logout']);

    Route::get('parent-options', [ApiController::class, 'getParentOption']);
    Route::get('active-options', [ApiController::class, 'getActiveOptionsFor']);
    Route::get('option', [ApiController::class, 'getOptionValue']);
    Route::get('banks', [ApiController::class, 'getBanks']);
    Route::get('branches', [ApiController::class, 'getBankBranches']);

    Route::post('save-application/{benefit_id}', [ApiController::class, 'saveApplication']);
    Route::get('validate-applicant', [ApiController::class, 'validateApplicant']);

    //Notifications
    Route::get('read-notifications', [ApiController::class, 'readNotificationPosts']);
    Route::get('unread-notifications', [ApiController::class, 'unReadNotificationPosts']);
    Route::get('open-notification-post/{notification_post_id}', [ApiController::class, 'openNotificationPost']);


    //Cases
    Route::get('cases', [ApiController::class, 'cases']);
    Route::get('case/{id}', [ApiController::class, 'getCase']);
    Route::post('save-case', [ApiController::class, 'saveCase']);
    Route::post('delete-case', [ApiController::class, 'deleteCase']);


    //Surveys
    Route::get('my-surveys', [ApiController::class, 'mySurveys']);
    Route::get('survey/{survey_id}', [ApiController::class, 'getSurveyForm']);
    Route::post('survey/{survey_id}/save', [ApiController::class, 'saveEntry']);
    Route::get('survey/{survey_id}/entry', [ApiController::class, 'getSurveyFormEntry']);


    //ADMIN ROUTES
    Route::get('dashboard', [AdminApiController::class, 'dashboardStatistics']);

});

//Cases
Route::get('get-deployment-settings/{deployment_code}', [DeploymentApiController::class, 'getDeploymentSetting']);
Route::get('deployment-has-subscription/{deployment_code}', [DeploymentApiController::class, 'hasActiveSubscription']);
Route::get('get-deployment-modules/{deployment_code}/{names?}', [DeploymentApiController::class, 'deploymentModules']);

//Location Endpoints
Route::get('get-country/{id}', [LocationApiController::class, 'getCountry']);
Route::get('get-state/{id}', [LocationApiController::class, 'getState']);
Route::get('get-county-district/{id}', [LocationApiController::class, 'getCounty']);
Route::get('get-city/{id}', [LocationApiController::class, 'getCity']);
Route::get('get-area/{id}', [LocationApiController::class, 'getArea']);

Route::get('get-nationalities', [LocationApiController::class, 'getNationalities']);
Route::get('get-countries', [LocationApiController::class, 'getCountries']);
Route::get('get-country-states/{country_id}', [LocationApiController::class, 'getStatesOrRegions']);
Route::get('get-state-counties/{state_id}', [LocationApiController::class, 'getDistrictsOrCounties']);
Route::get('get-cities/{county_id}', [LocationApiController::class, 'getDistrictsCities']);
Route::get('get-city-areas/{city_id}', [LocationApiController::class, 'getCityAreas']);
Route::get('get-location-clone/{tablename}', [LocationApiController::class, 'getLocationClone']);

//Merchants
Route::get('get-merchants/{merchant?}', [MerchantsApiController::class, 'getMerchants']);
Route::get('get-all-merchant-branches', [MerchantsApiController::class, 'getAllActiveMerchantBranches']);
Route::get('get-merchant-branches/{merchant_id}', [MerchantsApiController::class, 'getMerchantBranchesByMerchant']);
Route::get('get-merchant-countries', [MerchantsApiController::class, 'getMerchantCountries']);
Route::get('get-merchant-types', [MerchantsApiController::class, 'getMerchantTypes']);
Route::get('get-merchant-by-type/{type}/{country_id?}', [MerchantsApiController::class, 'getMerchantByType']);
Route::get('get-types-by-country/{country_id}', [MerchantsApiController::class, 'getTypesUnderCountry']);
Route::get('get-types-only-country/{country_id}', [MerchantsApiController::class, 'getTypesOnlyUnderCountry']);
Route::get('get-merchant/{merchant_id}', [MerchantsApiController::class, 'getMerchantById']);
Route::get('get-branch/{branch_id}', [MerchantsApiController::class, 'getBranchById']);
Route::get('get-merchant-clone/{tablename}', [MerchantsApiController::class, 'getMerchantClone']);

//Benefit Types
Route::get('get-benefit-types-clone/{tablename}', [BenefitTypesApiController::class, 'getBenefitTypesClone']);

Route::get('get-lookup-clone/{tablename}', [LookupApiController::class, 'getLookupClone']);
