<?php

use Illuminate\Support\Facades\Route;
use \App\Http\Controllers\SettingController;
use \App\Http\Controllers\StageEmailsController;
use \App\Http\Controllers\EmailsController;
/*
|--------------------------------------------------------------------------
| User Account Profile Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::middleware(['auth'])->name('settings.')->group(function () {
    Route::prefix('settings')->group(function () {

        Route::group(['middleware' => ['system-user']], function() {
            Route::get('/config', [SettingController::class, 'setConfig'])->name('setConfig')
                ->middleware(['check-permissions:administrator']);
            Route::get('/clear-bg-image', function () {
                settings()->set(['app_background_image'  =>  '']);
                return back();
            })->name('removeBgImage')
                ->middleware(['check-permissions:administrator']);
            Route::post('/application-config', [SettingController::class, 'saveConfig'])->name('saveConfig')
                ->middleware(['check-permissions:administrator']);
            Route::post('/terms-config', [SettingController::class, 'saveTerms'])->name('saveTerms')
                ->middleware(['check-permissions:administrator']);

            Route::get('/access-logs', [SettingController::class, 'accessLog'])->name('accessLog')
                ->middleware(['check-permissions:administrator,audit-logs']);
            Route::get('/force-logout/{user_id}/{log_id}', [SettingController::class, 'forceLogout'])->name('forceLogout')
                ->middleware(['check-permissions:administrator,audit-logs']);
        });

        Route::group(['middleware' => ['system-user:options']], function() {

            //Route for LOOKUP CODE
            Route::get('/lookup-codes', [SettingController::class, 'lookupCode'])->name('lookupCode');
            Route::post('/lookup-code/save', [SettingController::class, 'lookupCodeSave'])->name('lookupCodeSave');
            Route::get('/lookup-code/edit', [SettingController::class, 'lookupCodeEdit'])->name('lookupCodeEdit');
            Route::put('/lookup-code/update', [SettingController::class, 'lookupCodeUpdate'])->name('lookupCodeUpdate');

            //Route for LOOKUP OPTIONS
            Route::get('/lookup', [SettingController::class, 'lookup'])->name('lookup');
            Route::post('/lookup/save', [SettingController::class, 'lookupSave'])->name('lookupSave');
            Route::get('/lookup/edit', [SettingController::class, 'lookupEdit'])->name('lookupEdit');
            Route::put('/lookup/update', [SettingController::class, 'lookupUpdate'])->name('lookupUpdate');

        });

//        Route::group(['middleware' => ['check-permissions:administrator,country']], function() {
//            //Country
//            Route::get('/countries', [SettingController::class, 'countries'])->name('countries');
//            Route::post('/save-country', [SettingController::class, 'saveCountry'])->name('saveCountry');
//            Route::get('/country-status', [SettingController::class, 'countryStatus'])->name('countryStatus');
//            Route::get('/country-data', [SettingController::class, 'countryData'])->name('countryData');
//
//            //County
//            Route::get('/country/{id}/counties', [SettingController::class, 'counties'])->name('counties');
//            Route::post('/save-county', [SettingController::class, 'saveCounty'])->name('saveCounty');
//            Route::get('/county-status', [SettingController::class, 'countyStatus'])->name('countyStatus');
//        });

//        //ROUTES FOR EMAIL TEMPLATE CUSTOMIZER
//        Route::get('email-templates', [EmailsController::class,'index'])->name('email_templates');
//        Route::post('email-templates', [EmailsController::class,'save'])->name('email_templates.save');
//        Route::get('email-template/{id}/edit', [EmailsController::class,'edit'])->name('email_templates.edit');
//        Route::get('email-template/create', [EmailsController::class,'create'])->name('email_templates.create');
//        Route::put('email-template/{id}/update', [EmailsController::class,'update'])->name('email_templates.update');
//        Route::delete('email-template/destroy/{id}', [EmailsController::class,'destroy'])->name('email_templates.destroy');
//
//        //ROUTES FOR STAGE EMAILS
//        Route::get('stage-emails', [StageEmailsController::class,'index'])->name('stage_emails');
//        Route::post('stage-emails', [StageEmailsController::class,'save'])->name('stage_emails.save');
//        Route::get('stage-email/edit', [StageEmailsController::class,'edit'])->name('stage_emails.edit');
//        Route::get('stage-email/create', [StageEmailsController::class,'create'])->name('stage_emails.create');
//        Route::put('stage-email/{id}/update', [StageEmailsController::class,'update'])->name('stage_emails.update');
//        Route::delete('stage-email/destroy/{id}', [StageEmailsController::class,'destroy'])->name('stage_emails.destroy');

    });
});
