<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Auth\LoginController;
use App\Http\Controllers\Auth\RegisterController;
use App\Http\Controllers\Auth\ForgotPasswordController;
use App\Http\Controllers\Auth\ResetPasswordController;
use App\Http\Controllers\Auth\ExpiredPasswordController;
use App\Http\Controllers\UsersController;
use App\Http\Controllers\DashboardController;
use App\Http\Controllers\AuditController;
use App\Http\Controllers\SettingController;
use App\Http\Controllers\PagesContentController;
use App\Http\Controllers\Auth\CustomerLoginController;
use App\Http\Controllers\Auth\CustomerRegisterController;
use App\Http\Controllers\BenefitsController;
use App\Http\Controllers\MdataController;
use App\Http\Controllers\MerchantController;
use App\Http\Controllers\OnBoardingController;
use App\Models\Benefit;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Artisan;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::get('locale/{locale}', function ($locale) {
    session()->put('locale', $locale);
    App::setLocale(session()->get('locale'));
    return back();
})->name('language');

Route::controller(\App\Http\Controllers\Auth\SocialMediaConnections::class)->group(function (){
//    Route::get('auth/{social_media}/redirect', 'redirect')->name('socialRedirect');
    Route::get('/auth/google-redirect/{sign_type?}', 'redirect')->name('socialRedirect');
    Route::get('/auth/google-callback', 'callback')->name('socialCallback');
//    Route::get('auth/{social_media}/callback', 'callback')->name('socialCallback');
});

$applicant = config('app.portal_base_url','portal');

Route::get('tomorrow', function () {
    if(1686155636 < strtotime(now())) {
        return 'Success';
    }
    return strtotime(now()).' => '.strtotime("-24 hours");
});

Route::get('send-mail', function () {
    $subject = "Email Subject";
    $emails = ['alfred.sowa@bewsys.com'];
    $ccs = ['blackmanlegacyghana@gmail.com'];
    $user = \App\Models\User::find(1);

    // Ship the order...

    Mail::to($emails)->send(new \App\Mail\OnBoarding($user));

});

Route::get('resetting', function () {
    Artisan::call('clear-compiled');
    Artisan::call('cache:clear');
    Artisan::call('view:clear');
    Artisan::call('config:clear');
    return back();
});

Route::get('ss-link', function () {
    Artisan::call('storage:link');
    return back();
});

Route::view('/','home')->name('index');
Route::redirect('/','/dashboard');

//Route::view('/update-complete','update-complete')->name('updateComplete');
Route::view('/trial-expired','welcome')->name('trail_expired');
Route::view('/suspended','suspended')->name('suspended');
Route::view('/ended','ended')->name('ended');

//Login Routes...
Route::controller(LoginController::class)->group(function (){
    Route::get('login', 'showLoginForm')->name('login');
    Route::post('login', 'login')->name('login');
    Route::post('logout', 'logout')->name('logout');
});

// Registration Routes...
Route::controller(RegisterController::class)->group(function () {
    Route::get('signup/{user_type?}', 'signUp')->name('signUp');
    Route::get('register', 'showRegistrationForm')->name('register');
    Route::post('register', 'register');
});

// Password Reset Routes...
Route::controller(ForgotPasswordController::class)->group(function () {
    Route::get('password/reset', 'showLinkRequestForm')->name('password.request');
    Route::post('password/email', 'sendResetLinkEmail')->name('password.email');
});
Route::controller(ResetPasswordController::class)->group(function (){
    Route::get('password/reset/{token}', 'showResetForm')->name('password.reset');
//    Route::get('password/reset', 'showResetForm');
    Route::post('password/reset', 'reset')->name('password.update');
});

//Password Expiry Routes
Route::controller(ExpiredPasswordController::class)->group(function () {
    Route::get('password/expired', 'expired')->name('password.expired');
    Route::put('password/post_expired', 'postExpired')->name('password.post_expired');
});

Route::get('download', function (\Illuminate\Http\Request $request) {
    return \Illuminate\Support\Facades\Storage::download($request->file_path);
})->name('download-file');

Route::prefix($applicant)->name('customer.')->group(function () use ($applicant) {

    Route::controller(CustomerRegisterController::class)->group(function () use ($applicant) {
        Route::get('signup', 'customerSignupForm')->name('signup');
        Route::post('signup', 'create')->name('create');
        Route::get('verify-account', 'verify')->name('verify');
        Route::post('verify-account', 'verifyCode')->name('verifyCode');
    });

    Route::controller(CustomerLoginController::class)->group(function () use ($applicant) {
        Route::get('login', 'customerLoginForm')->name('login');
        Route::post('login', 'login')->name('access');
        Route::post('logout', 'logout')->name('logout');
        Route::get('forgot-password/{phone?}/{reset_password_token?}', 'forgotPassword')->name('forgotPassword');
        Route::post('forgot-password', 'forgotPasswordSet')->name('forgotPasswordSet');
    });

});

Route::middleware(['auth'])->group(function () use ($applicant) {
    /**
     * Verification Routes
     */
    Route::get('/email/verify', 'VerificationController@show')->name('verification.notice');
    Route::get('/email/verify/{id}/{hash}', 'VerificationController@verify')->name('verification.verify')->middleware(['signed']);
    Route::post('/email/resend', 'VerificationController@resend')->name('verification.resend');
});

//Basic General Routes
Route::get('/get-types-only-country/{merchant_country}', function (string $merchant_country) {
//    return $merchant_country;
        return getMerchantTypesOnlyByCountry($merchant_country);
});

Route::middleware(['auth','verified'])->group(function () use ($applicant) {


    // ON-BOARDING ROUTES
    Route::controller(OnBoardingController::class)->prefix('on-boarding')->name('on_boarding.')->group(function () {
        Route::get('/start', 'index')->name('index');
        Route::get('/setup-organization', 'organizationSetup')->name('client');
        Route::get('/setup-personal-details', 'personalDetails')->name('personalDetails');
        Route::get('/setup-project', 'setupProject')->name('setupProject');
        Route::get('/setup-meeting', 'setupMeeting')->name('setupMeeting');
        Route::get('/selected-meeting-date', 'selectedMeetingDate')->name('selectedMeetingDate');
        Route::post('/create', 'create')->name('create');
        Route::post('/update-profile', 'update_profile')->name('update_profile');
        Route::post('/update-project', 'updateProject')->name('updateProject');
        Route::get('/verify-account/{user_id}', 'verifyAccount')->name('verifyAccount');
        Route::get('/on-boarding-tour', 'locationPopoverContent')->name('locationPopoverContent');
        Route::get('/skip-tour', 'skipTour')->name('skipTour');
    });

    Route::controller(\App\Http\Controllers\DeploymentController::class)
        ->name('subscription.')->prefix('subscription')->group(function () {
            Route::get('settings/{client_id}/{deployment_id?}', 'projectSettings')->name('projectSettings');
            Route::post('settings', 'saveProjectSettings')->name('saveProjectSettings');
            Route::post('save-terms', 'saveTerms')->name('saveTerms');
            Route::post('settings/code-generation/{deployment}', 'saveApplicantGenCode')->name('saveApplicantGenCode');
            Route::get('check-subdomain', 'checkSubdomain')->name('checkSubdomain');
            Route::get('update/{id}', 'updateProject')->name('updateProject');
        });

    /** Client Routes */
    require __DIR__.'/client.php';

    /** Customer Routes */
    require __DIR__ . '/membership.php';

    Route::middleware(['admin-customer'])->group(function () use($applicant) {
        /** Benefit Registration Routes */
        require __DIR__.'/applicant.php';

        /** Cases Routes */
        require __DIR__.'/cases.php';
    });

    //Main Admin Routes
    Route::group(['middleware' => ['admin-access']], function() {
        Route::get('flush-expired-subscriptions', function () {
            Artisan::call('flush:expired-subscriptions');
            notify("All expired subscriptions have been flushed out.",'success','Successful!');
            return back();
        })->name('flush_expired_subscriptions');

        //Client Routes
        Route::controller(\App\Http\Controllers\ClientController::class)->name('clients.')->group(function () {
            Route::get('deployments', 'index')->name('index');
            Route::get('get-deployment/{id}', 'getClient')->name('getClient');
            Route::get('deployment/ownership/{id}', 'getOwner')->name('getOwner');
            Route::post('saveInfo', 'saveInfo')->name('saveInfo');
            Route::post('changeOwner', 'changeOwner')->name('changeOwner');
            Route::post('new-client', 'newClient')->name('newClient');
            Route::post('client-approval', 'clientApproval')->name('clientApproval');
            Route::post('remove-from-client', 'removeMe')->name('removeMe');
        });

        /** Household Routes */
        require __DIR__.'/household.php';

        /** Program Routes */
        require __DIR__.'/program.php';

        /** Partners Routes */
        require __DIR__.'/partners.php';

        /** Account Routes */
        require __DIR__.'/account.php';

        /** User Profile Routes */
        require __DIR__.'/account-profile.php';

        /** User Management Routes */
        require __DIR__.'/user-management.php';

        /** Benefits Management Routes */
        require __DIR__.'/benefits.php';

        /** Benefit Registration Routes */
        require __DIR__.'/benefit-registration.php';

        /** Bank Management Routes */
        require __DIR__ . '/merchant.php';

        /** Member Records Routes */
        require __DIR__.'/member-records.php';

        /** Office Management Routes */
        require __DIR__.'/offices.php';

        /** Preferences Routes */
        require __DIR__.'/instances.php';

        /** Settings Management Routes */
        require __DIR__.'/settings.php';

        /** Settings Payment Routes */
        require __DIR__.'/payment.php';

        /** Settings Projects Routes */
        require __DIR__.'/projects.php';

        /** Notification Routes */
        require __DIR__ . '/announcements.php';

        /** Report Routes */
        require __DIR__.'/report.php';

        /** Survey Routes */
        require __DIR__.'/survey.php';

        /** Survey Module Routes */
        require __DIR__.'/survey-form.php';

        /** Team Routes */
        require __DIR__.'/teams.php';

        /** Subscription Routes */
        require __DIR__.'/subscription.php';

        /** Household Routes */
        require __DIR__.'/custom-fields.php';

        if (true) {
            /** Subscription Routes */
            require __DIR__.'/plans.php';

            //System Audit Routes
            Route::controller(AuditController::class)->group(function () {
                Route::get('system_audit_trail', 'index')->name('system.audit.trail')->middleware(['check-permissions:administrator,audit-logs']);
            });

            //Settings Routes
            Route::controller(SettingController::class)->group(function () {
                Route::get('application-settings', 'index')->name('app.settings');
                Route::get('get-country', 'countryData')->name('countryData');
            });

            //Settings Routes
            Route::controller(SettingController::class)->group(function () {
                Route::get('application-settings', 'organisationSetup')->name('organisationSetup');
            });

            $pages_list = ['faq', 'contact_us', 'about_us','downloads'];

            Route::controller(PagesContentController::class)->group(function () use($pages_list) {
                foreach ($pages_list as $page) {
                    Route::get('/page_content_' . $page,  $page)->name('page.' . $page);
                    Route::post('/store_content_' . $page,  'store_'.$page)->name('store.page.' . $page);
                }
                Route::get('/page_faq_edit/{faq}',  'editFAQDetails')->name('page.edit.faq');
                Route::get('/page_faq_delete/{faq}',  'deleteFAQDetails')->name('page.delete.faq');

            });
        }

    });

    //Dashboard Routes
    Route::controller(DashboardController::class)->group(function () {
        Route::get('/dashboard', 'index')->name('dashboard')->middleware(['check-permissions:administrator,view-dashboard']);
    });

    Route::controller(DashboardController::class)->group(function () {
        Route::get('/program_dashboard', 'program_dashboard')->middleware(['check-permissions:administrator,view-dashboard']);
    });

    Route::group(['middleware' => ['auth', 'active_user']], function() {

        //User Management Routes
        Route::name('name.')->controller(UsersController::class)->group(function () {
            Route::get('profile', 'profile')->name('profile');
            Route::get('profile/{id}', 'edit_profile')->name('profile.edit');
            Route::put('profile', 'save_profile')->name('profile.save');
            Route::get('change-password', 'password')->name('password');
            Route::put('change-password-save', 'save_password')->name('password.save');
            Route::put('user/update_user_profile_picture', 'update_user_profile_picture')->name('update_user_picture');
        });

        Route::resource('users', 'UsersController');
        Route::post('update-user-profile', [UsersController::class, 'save_profile'])
            ->name('user.profile');
        Route::post('user-update-profile-picture', [UsersController::class, 'update_user_profile_picture'])
            ->name('update_user_picture');
        Route::post('user-update-profile-picture', [UsersController::class, 'update_user_profile_picture'])
            ->name('update_user_picture');
        Route::get('user-save-password', [UsersController::class, 'save_password'])
            ->name('user.password.save');
        Route::get('user-activation', [UsersController::class, 'activation'])
            ->name('user_activation');
        Route::resource('roles', 'RolesController');

        Route::get('settings/branding', [UsersController::class, 'user_branding'])->name('user_branding');
        Route::post('branding/{user_id}', [UsersController::class, 'set_user_branding'])->name('user_set_branding');

        Route::get('settings/import-beneficiaries', [UsersController::class, 'import_beneficiaries'])->name('beneficiaries.import');
        Route::post('settings/import-beneficiaries', [UsersController::class, 'handle_beneficiaries_import2'])->name('handle_beneficiary_import');

    });

    Route::get('/settings/mdata', [MdataController::class, 'index'])->middleware(['auth', 'admin-access'])->name('mdata.index');
    Route::post('/settings/import-mdata', [MdataController::class, 'import2'])->middleware(['auth', 'admin-access'])->name('mdata.import');
    Route::post('/settings/import-mdata-village', [MdataController::class, 'create_districts'])->middleware(['auth', 'admin-access'])->name('villages.import');

    Route::get('/settings/inventory', [MdataController::class, 'inventory'])->name('inventory.index');
    Route::post('/settings/inventory', [MdataController::class, 'create_inventory_item'])->name('inventory.create-item');
    Route::get('/get_inventory_item/{item_id}', [MdataController::class, 'get_inventory_item']);
    Route::post('/settings/inventory/update/{item_id}', [MdataController::class, 'update_inventory_item']);

    


    //get regions of country
    Route::get('/get-country-regions/{country_id}', [SettingController::class, 'getCountryRegions']);
    //get regions of country
    Route::get('/get-country-villages/{country_id}', [SettingController::class, 'getCountryVillages']);
    //get subcounties of county
    Route::get('/get-sub-counties/{county_id}', [SettingController::class, 'getSubCounties']);
    Route::get('/get-villages/{subcounty_id}', [SettingController::class, 'getVillages']);
    //get counties of region
    Route::get('/get-region-counties/{region_id}', [SettingController::class, 'getRegionCounties']);
    //get merchant/banks by type
    Route::get('/get-banks-by-merchant-type/{merchant_type}', [MerchantController::class, 'getBanksByType']);

    Route::get('/get-user-deployments/{user_id}', [UsersController::class, 'get_users_deployments']);

    //get branches of a specific bank
    Route::get('/get-merchant-branches/{merchant_id}', [MerchantController::class, 'getBankBranchesByMerchantId']);
    Route::get('/get-coordinates', [UsersController::class, 'get_cord2']);

    //Save benefit payment record
    Route::post('/benefit-payment/save', [BenefitsController::class, 'saveBenefitPayment'])->name('benefit_payment.save');
    Route::get('/get-application/{application_id}', [BenefitsController::class, 'getApplication']);
    Route::get('/get-benefit-payment-details/{application_id}', [BenefitsController::class, 'getBenefitPayment']);

    Route::get('/get-inventory-items', [SettingController::class, 'get_inventory_items']);
    Route::get('/get-application-items/{application_id}', [SettingController::class, 'get_application_items']);
    Route::get('/get-application-issuance-records/{application_id}', [SettingController::class, 'get_issuance_records']);

    Route::get('/get-issuance-mode-id/{benefit_financial}', [SettingController::class, 'getIssuanceModeId']);

    Route::get('/custom/beneficiaries', [BenefitsController::class, 'custom_beneficiaries']);
    Route::get('/get_constituency/{constituency_id}', [BenefitsController::class, 'get_constituency']);
    Route::get('/get_district/{district_id}', [BenefitsController::class, 'get_district']);

});
// });

